/****************************************************************************
**
** Copyright (C) 2007-2008 Trolltech ASA. All rights reserved.
**
** This file is part of the QGtkStyle project on Trolltech Labs.
**
** This file may be used under the terms of the GNU General Public
** License version 2.0 as published by the Free Software Foundation
** and appearing in the file LICENSE.GPL included in the packaging of
** this file.  Please review the following information to ensure GNU
** General Public Licensing requirements will be met:
** http://www.trolltech.com/products/qt/opensource.html
**
** If you are unsure which license is appropriate for your use, please
** review the following information:
** http://www.trolltech.com/products/qt/licensing.html or contact the
** sales department at sales@trolltech.com.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
****************************************************************************/

#include "qgtkstyle.h"
#include <QStatusBar>
#include <QLibrary>
#include <QWidget>
#include <QApplication>
#include <QStyleOption>
#include <QPushButton>
#include <QPainter>
#include <QMainWindow>
#include <QToolBar>
#include <QMenu>
#include <QHeaderView>
#include <QMenuBar>
#include <QComboBox>
#include <QSpinBox>
#include <QScrollBar>
#include <QAbstractButton>
#include <QToolButton>
#include <QSettings>
#include <QGroupBox>
#include <QRadioButton>
#include <QCheckBox>
#include <QTreeView>
#include <qpixmapcache.h>
#include <qdebug.h>
#undef signals
#include "gtksymbols_p.h"


static bool usePixmapCache = true;

static const char * const dock_widget_close_xpm[] = {
    "11 13 5 1",
    " 	c None",
    ".	c #D5CFCB",
    "+	c #6C6A67",
    "@	c #6C6A67",
    "$	c #B5B0AC",
    "           ",
    " @@@@@@@@@ ",
    "@+       +@",
    "@ +@   @+ @",
    "@ @@@ @@@ @",
    "@  @@@@@  @",
    "@   @@@   @",
    "@  @@@@@  @",
    "@ @@@ @@@ @",
    "@ +@   @+ @",
    "@+       +@",
    " @@@@@@@@@ ",
    "           "};

static const char * const dock_widget_restore_xpm[] = {
    "11 13 5 1",
    " 	c None",
    ".	c #D5CFCB",
    "+	c #6C6A67",
    "@	c #6C6A67",
    "#	c #6C6A67",
    "           ",
    " @@@@@@@@@ ",
    "@+       +@",
    "@   #@@@# @",
    "@   @   @ @",
    "@ #@@@# @ @",
    "@ @   @ @ @",
    "@ @   @@@ @",
    "@ @   @   @",
    "@ #@@@@   @",
    "@+       +@",
    " @@@@@@@@@ ",
    "           "};

// from windows style
static const int windowsItemFrame        =  2; // menu item frame width
static const int windowsItemHMargin      =  3; // menu item hor text margin
static const int windowsItemVMargin      = 26; // menu item ver text margin
static const int windowsRightBorder      = 15; // right border on windows
static const int groupBoxBottomMargin    =  2;  // space below the groupbox
static const int groupBoxTitleMargin     =  6;  // space between contents and title
static const int groupBoxTopMargin       =  2;  // space between contents and title

#define CL_MAX(a,b) (a)>(b) ? (a):(b) // ### qMin/qMax does not work for vc6
#define CL_MIN(a,b) (a)<(b) ? (a):(b) // remove this when it is working

#undef GTK_OBJECT_FLAGS
#define GTK_OBJECT_FLAGS(obj)(((GtkObject*)(obj))->flags)

// To recover alpha we apply the gtk painting function two times to
// white, and black window backgrounds. This can be used to
// recover the premultiplied alpha channel

#if Q_BYTE_ORDER == Q_BIG_ENDIAN
#   define RED 3
#   define GREEN 2
#   define BLUE 1
#   define ALPHA 0
#else
#   define RED 0
#   define GREEN 1
#   define BLUE 2
#   define ALPHA 3
#endif

#define DRAW_WITH_ALPHA(draw_func) \
    if (rect.width() > QWIDGETSIZE_MAX || rect.height() > QWIDGETSIZE_MAX) \
        return; \
    cache = QPixmap(rect.size()); \
    cache.fill(Qt::transparent); \
    QRect pixmapRect(0, 0, rect.width(), rect.height()); \
    QPainter cachePainter(&cache); \
    { \
        GdkPixmap *pixmap = gdk_pixmap_new(GDK_DRAWABLE(window->window), rect.width(), rect.height(), -1); \
        if (!pixmap) \
            return; \
        style = p_gtk_style_attach (style, window->window); \
        gdk_draw_rectangle(pixmap, style->black_gc, true, 0, 0, rect.width(), rect.height()); \
        draw_func; \
        GdkPixbuf *imgb = gdk_pixbuf_new(GDK_COLORSPACE_RGB, true, 8, rect.width(), rect.height()); \
        if (!imgb) \
            return; \
        imgb = gdk_pixbuf_get_from_drawable(imgb, pixmap, NULL, 0, 0, 0, 0, rect.width(), rect.height()); \
        gdk_draw_rectangle(pixmap, style->white_gc, true, 0, 0, rect.width(), rect.height()); \
        draw_func; \
        GdkPixbuf *imgw = gdk_pixbuf_new(GDK_COLORSPACE_RGB, true, 8, rect.width(), rect.height()); \
        if (!imgw) \
            return; \
        imgw = gdk_pixbuf_get_from_drawable(imgw, pixmap, NULL, 0, 0, 0, 0, rect.width(), rect.height()); \
        uchar* wdata = (uchar*)gdk_pixbuf_get_pixels(imgw); \
        uchar* bdata = (uchar*)gdk_pixbuf_get_pixels(imgb); \
        int index = 0; \
        for ( int y = 0 ; y < rect.height() ; ++y) { \
            for ( int x = 0 ; x < rect.width() ; ++x) { \
               int alpha = qMax(bdata[index + RED] - wdata[index + RED], bdata[index + GREEN] - wdata[index + GREEN]); \
               alpha = qMax(alpha, bdata[index + BLUE] - wdata[index + BLUE]) + 255; \
               uchar val = bdata[index + RED]; \
               bdata[index + ALPHA] = alpha; \
               bdata[index + RED] = bdata[index + BLUE]; \
               bdata[index + BLUE] = val; \
               index += 4; \
            } \
        } \
        QImage converted( (const uchar*)bdata, rect.width(), rect.height(), QImage::Format_ARGB32_Premultiplied); \
        cachePainter.drawImage(QPoint(0, 0), converted); \
        gdk_drawable_unref(pixmap); \
        gdk_pixbuf_unref(imgw); \
        gdk_pixbuf_unref(imgb); \
    }

#define DRAW_WITHOUT_ALPHA(draw_func) \
    cache = QPixmap(rect.size()); \
    QRect pixmapRect(0, 0, rect.width(), rect.height()); \
    QPainter cachePainter(&cache); \
    { \
        GdkPixmap *pixmap = gdk_pixmap_new(GDK_DRAWABLE(window->window), rect.width(), rect.height(), -1); \
        style = p_gtk_style_attach (style, window->window); \
        gdk_draw_rectangle(pixmap, *style->bg_gc, true, 0, 0, rect.width(), rect.height()); \
        draw_func; \
        GdkPixbuf *imgb = gdk_pixbuf_new(GDK_COLORSPACE_RGB, true, 8, rect.width(), rect.height()); \
        imgb = gdk_pixbuf_get_from_drawable(imgb, pixmap, NULL, 0, 0, 0, 0, rect.width(), rect.height()); \
        uchar* bdata = (uchar*)gdk_pixbuf_get_pixels(imgb); \
        int index = 0; \
        for ( int y = 0 ; y < rect.height() ; ++y) { \
            for ( int x = 0 ; x < rect.width() ; ++x) { \
               uchar val = bdata[index]; \
               bdata[index] = bdata[index+2]; \
               bdata[index+2] = val; \
               index+=4; \
            } \
        } \
        QImage converted( (const uchar*)bdata, rect.width(), rect.height(), QImage::Format_RGB32); \
        cachePainter.drawImage(QPoint(0, 0), converted); \
        gdk_drawable_unref(pixmap); \
        gdk_pixbuf_unref(imgb); \
    }

static QColor mergedColors(const QColor &colorA, const QColor &colorB, int factor = 50)
{
    const int maxFactor = 100;
    QColor tmp = colorA;
    tmp.setRed((tmp.red() * factor) / maxFactor + (colorB.red() * (maxFactor - factor)) / maxFactor);
    tmp.setGreen((tmp.green() * factor) / maxFactor + (colorB.green() * (maxFactor - factor)) / maxFactor);
    tmp.setBlue((tmp.blue() * factor) / maxFactor + (colorB.blue() * (maxFactor - factor)) / maxFactor);
    return tmp;
}

static GtkStateType gtkState(const QStyleOption *option)
{
    GtkStateType state = GTK_STATE_NORMAL;
    if (!(option->state & QStyle::State_Enabled))
        state = GTK_STATE_INSENSITIVE;
    else if (option->state & QStyle::State_MouseOver)
        state = GTK_STATE_PRELIGHT;
    return state;
}

GtkStyle *get_style(GtkWidget *gtkWidget)
{
    Q_ASSERT(gtkWidget);
    GtkStyle* style = gtkWidget->style;
    Q_ASSERT(style);
    return style;
}

static QString uniqueName(const QString &key, GtkStateType state, GtkShadowType shadow, const QSize &size, GtkWidget *widget = 0)
{
    QString tmp;
    tmp.sprintf("%s-%d-%d-%dx%d-%p", key.toLatin1().constData(), uint(state), uint(shadow), size.width(),
                size.height(), widget);
    return tmp;
}

// Note currently painted without alpha for performance reasons
static void qgtkstyle_paint_box_gap(const GtkWidget *window, GtkWidget *gtkWidget, const gchar* part, const QRect &rect,
                                      GtkStateType state, GtkShadowType shadow, GtkPositionType gap_side, gint x, gint width,
                                      GtkStyle *style, QPainter *painter)
{
    if (!rect.isValid())
        return;
    QPixmap cache;
    QString pixmapName = uniqueName(part, state, shadow, rect.size(), gtkWidget) + 's' + gap_side + 'w' + width + 'g' + x;
    if (!usePixmapCache || !QPixmapCache::find(pixmapName, cache)) {
        DRAW_WITHOUT_ALPHA(p_gtk_paint_box_gap (style, pixmap, state, shadow, NULL,
                                 gtkWidget, (gchar*)part, 0, 0, rect.width(), rect.height(), gap_side, x, width))
        if (usePixmapCache)
            QPixmapCache::insert(pixmapName, cache);
    }
    painter->drawPixmap(rect.topLeft(), cache);

}

static void qgtkstyle_paint_box(const GtkWidget *window, GtkWidget *gtkWidget, const gchar* part,
                                  const QRect &rect, GtkStateType state, GtkShadowType shadow, GtkStyle *style,
                                  QPainter *painter, const QString &pmKey = QString(), bool alpha = true)
{
    if (!rect.isValid())
        return;
    QPixmap cache;
    QString pixmapName = uniqueName(part, state, shadow, rect.size(), gtkWidget) + pmKey;
    if (!usePixmapCache || !QPixmapCache::find(pixmapName, cache)) {    
        if (alpha) {
            DRAW_WITH_ALPHA(p_gtk_paint_box (style, pixmap, state, shadow, NULL, gtkWidget, part, 0, 0, rect.width(), rect.height()));
        }
        else {
            DRAW_WITHOUT_ALPHA(p_gtk_paint_box (style, pixmap, state, shadow, NULL, gtkWidget, part, 0, 0, rect.width(), rect.height()));
        }
        if (usePixmapCache)
            QPixmapCache::insert(pixmapName, cache);
    }
    painter->drawPixmap(rect.topLeft(), cache);
}

static void qgtkstyle_paint_hline(const GtkWidget *window, GtkWidget *gtkWidget, const gchar* part,
                                  const QRect &rect, GtkStateType state, GtkStyle *style,
                                  QPainter *painter, int x1, int x2, int y, const QString &pmKey = QString())
{
    if (!rect.isValid())
        return;
    QPixmap cache;
    QString pixmapName = uniqueName(part, state, GTK_SHADOW_NONE, rect.size(), gtkWidget) + x1 + x2 + y + pmKey;
    if (!usePixmapCache || !QPixmapCache::find(pixmapName, cache)) {
        DRAW_WITH_ALPHA(p_gtk_paint_hline (style, pixmap, state, NULL, gtkWidget, part, x1, x2, y));
        if (usePixmapCache)
            QPixmapCache::insert(pixmapName, cache);
    }
    painter->drawPixmap(rect.topLeft(), cache);
}

static void qgtkstyle_paint_expander(const GtkWidget *window, GtkWidget *gtkWidget, const gchar* part,
                                       const QRect &rect, GtkStateType state, GtkExpanderStyle expander_state, GtkStyle *style,
                                       QPainter *painter, const QString &pmKey = QString())
{
    if (!rect.isValid())
        return;
    QPixmap cache;
    QString pixmapName = uniqueName(part, state, GTK_SHADOW_NONE, rect.size(), gtkWidget) + expander_state + pmKey;
    if (!usePixmapCache || !QPixmapCache::find(pixmapName, cache)) {
        DRAW_WITH_ALPHA(p_gtk_paint_expander (style, pixmap, state, NULL, gtkWidget, part, 5, 5, expander_state));
        if (usePixmapCache)
            QPixmapCache::insert(pixmapName, cache);
    }
    painter->drawPixmap(rect.topLeft(), cache);
}

/*
static void qgtkstyle_render_icon(const GtkWidget *window, GtkWidget *gtkWidget, const char* part, const GtkIconSource* source, GtkTextDirection direction, 
                                       GtkStateType state, const QRect &rect, GtkIconSize size, GtkStyle *style,
                                       QPainter *painter, const QString &pmKey = QString())
{
    if (!rect.isValid())
        return;
    QPixmap cache;
    QString pixmapName = uniqueName(part, state, GTK_SHADOW_NONE, rect.size(), gtkWidget) + pmKey;
    if (!usePixmapCache || !QPixmapCache::find(pixmapName, cache)) {
        DRAW_WITH_ALPHA(gtk_style_render_icon (style, source, direction, state, size, gtkWidget, part));
        if (usePixmapCache)
            QPixmapCache::insert(pixmapName, cache);
    }
    painter->drawPixmap(rect.topLeft(), cache);
}
*/

static void qgtkstyle_paint_focus(const GtkWidget *window, GtkWidget *gtkWidget, const gchar* part,
                                    const QRect &rect, GtkStateType state, GtkStyle *style,
                                    QPainter *painter, const QString &pmKey = QString())
{
    if (!rect.isValid())
        return;
    QPixmap cache;
    QString pixmapName = uniqueName(part, state, GTK_SHADOW_NONE, rect.size(), gtkWidget) + pmKey;
    if (!usePixmapCache || !QPixmapCache::find(pixmapName, cache)) {
        DRAW_WITH_ALPHA(p_gtk_paint_focus (style, pixmap, state, NULL, gtkWidget, part, 0, 0, rect.width(), rect.height()));
        if (usePixmapCache)
            QPixmapCache::insert(pixmapName, cache);
    }
    painter->drawPixmap(rect.topLeft(), cache);
}


static void qgtkstyle_resize_grip(const GtkWidget *window, GtkWidget *gtkWidget, const gchar* part,
                                    const QRect &rect, GtkStateType state, GtkShadowType shadow, GdkWindowEdge edge, GtkStyle *style,
                                    QPainter *painter, const QString &pmKey = QString())
{
    if (!rect.isValid())
        return;

    QPixmap cache;
    QString pixmapName = uniqueName(part, state, shadow, rect.size(), gtkWidget) + pmKey;
    if (!usePixmapCache || !QPixmapCache::find(pixmapName, cache)) {
        DRAW_WITH_ALPHA(p_gtk_paint_resize_grip (style, pixmap, state, NULL, gtkWidget, part, edge, 0, 0, rect.width(), rect.height()));
        if (usePixmapCache)
            QPixmapCache::insert(pixmapName, cache);
    }
    painter->drawPixmap(rect.topLeft(), cache);
}

static void qgtkstyle_paint_arrow(const GtkWidget *window, GtkWidget *gtkWidget, const gchar* part,
                                    const QRect &rect, GtkArrowType arrow_type, GtkStateType state, GtkShadowType shadow,
                                    gboolean fill, GtkStyle *style, QPainter *painter)
{
    if (!rect.isValid())
        return;
    QPixmap cache;
    QString pixmapName = uniqueName(part, state, shadow, rect.size()) + (int)arrow_type;
    if (!usePixmapCache || !QPixmapCache::find(pixmapName, cache)) {
        DRAW_WITH_ALPHA(p_gtk_paint_arrow (style, pixmap, state, shadow, NULL, gtkWidget, part,
                                    arrow_type, fill, 0, 0, rect.width(), rect.height()))
        if (usePixmapCache)
            QPixmapCache::insert(pixmapName, cache);
    }
    painter->drawPixmap(rect.topLeft(), cache);
}

static void qgtkstyle_paint_handle (const GtkWidget *window, GtkWidget *gtkWidget, const gchar* part, const QRect &rect,
                                      GtkStateType state, GtkShadowType shadow, GtkOrientation orientation, GtkStyle *style, QPainter *painter)
{
    if (!rect.isValid())
        return;
    QPixmap cache;
    QString pixmapName = uniqueName(part, state, shadow, rect.size()) + orientation;
    if (!usePixmapCache || !QPixmapCache::find(pixmapName, cache)) {
        DRAW_WITH_ALPHA(p_gtk_paint_handle (style, pixmap, state, shadow, NULL, gtkWidget, part, 0, 0, rect.width(), rect.height(), orientation));
        if (usePixmapCache)
            QPixmapCache::insert(pixmapName, cache);
    }
    painter->drawPixmap(rect.topLeft(), cache);
}

static void qgtkstyle_paint_slider(const GtkWidget *window, GtkWidget *gtkWidget, const gchar* part, const QRect &rect, GtkStateType state, GtkShadowType shadow,
                                     GtkStyle *style, QPainter *painter, GtkOrientation orientation, const QString &pmKey = QString())
{
    if (!rect.isValid())
        return;
    QPixmap cache;
    QString pixmapName = uniqueName(part, state, shadow, rect.size(), gtkWidget) + pmKey;
    if (!usePixmapCache || !QPixmapCache::find(pixmapName, cache)) {
        DRAW_WITH_ALPHA(p_gtk_paint_slider (style, pixmap, state, shadow, NULL, gtkWidget, part, 0, 0, rect.width(), rect.height(), orientation));
        if (usePixmapCache)
            QPixmapCache::insert(pixmapName, cache);
    }
    painter->drawPixmap(rect.topLeft(), cache);
}


static void qgtkstyle_paint_shadow(const GtkWidget *window, GtkWidget *gtkWidget, const gchar* part, const QRect &rect, GtkStateType state, GtkShadowType shadow, GtkStyle *style, QPainter *painter, const QString &pmKey = QString())
{
    if (!rect.isValid())
        return;
    QRect r = rect;
    QPixmap cache;
    QString pixmapName = uniqueName("shadow", state, shadow, rect.size()) + pmKey;
    if (!usePixmapCache || !QPixmapCache::find(pixmapName, cache)) {
        DRAW_WITH_ALPHA(p_gtk_paint_shadow(style, pixmap, state, shadow, NULL, gtkWidget, part, 0, 0, rect.width(), rect.height()));
        if (usePixmapCache)
            QPixmapCache::insert(pixmapName, cache);
    }
    painter->drawPixmap(rect.topLeft(), cache);
}

static void qgtkstyle_paint_flat_box(const GtkWidget *window, GtkWidget *gtkWidget, const gchar* part, const QRect &rect, GtkStateType state,
                                       GtkShadowType shadow, GtkStyle *style, QPainter *painter)
{
    if (!rect.isValid())
        return;
    QRect r = rect;
    QPixmap cache;
    QString pixmapName = uniqueName(part, state, shadow, rect.size());
    if (!usePixmapCache || !QPixmapCache::find(pixmapName, cache))
    {
        DRAW_WITH_ALPHA(gtk_paint_flat_box (style, pixmap, state, shadow, NULL, gtkWidget, part, 0, 0, rect.width(), rect.height()));
        if (usePixmapCache)
            QPixmapCache::insert(pixmapName, cache);
    }
    painter->drawPixmap(rect.topLeft(), cache);
}

static void qgtkstyle_paint_extention(const GtkWidget *window, GtkWidget *gtkWidget, const gchar *part,
                                        const QRect &rect, GtkStateType state, GtkShadowType shadow,
                                        GtkPositionType gap_pos, GtkStyle *style, QPainter *painter)
{
    if (!rect.isValid())
        return;
    QRect r = rect;
    QPixmap cache;
    QString pixmapName = uniqueName(part, state, shadow, rect.size(), gtkWidget) + QString::number(gap_pos);
    if (!usePixmapCache || !QPixmapCache::find(pixmapName, cache)) {
        DRAW_WITH_ALPHA(p_gtk_paint_extension (style, pixmap, state, shadow, NULL, gtkWidget, (gchar*)part, 0, 0, rect.width(), rect.height(), gap_pos));
        if (usePixmapCache)
            QPixmapCache::insert(pixmapName, cache);
    }
    painter->drawPixmap(rect.topLeft(), cache);
}

static void qgtkstyle_paint_option(GtkWidget *window, const QRect &rect, GtkStateType state, GtkShadowType shadow, GtkStyle *style, QPainter *painter)
{
    if (!rect.isValid())
        return;
    QRect r = rect;
    QPixmap cache;
    QString pixmapName = uniqueName("option", state, shadow, rect.size());
    if (!usePixmapCache || !QPixmapCache::find(pixmapName, cache)) {
        DRAW_WITH_ALPHA(p_gtk_paint_option ( style, pixmap, state, shadow, NULL, window, "checkbutton", 0, 0, rect.width() - 1, rect.height() - 1));
        if (usePixmapCache)
            QPixmapCache::insert(pixmapName, cache);
    }
    painter->drawPixmap(rect.topLeft(), cache);
}

static void qgtkstyle_paint_checkbox(GtkWidget *window, const QRect &rect, GtkStateType state, GtkShadowType shadow, GtkStyle *style, QPainter *painter)
{
    if (!rect.isValid())
        return;
    QRect r = rect;
    QPixmap cache;
    QString pixmapName = uniqueName("checkbutton", state, shadow, rect.size());
    if (!usePixmapCache || !QPixmapCache::find(pixmapName, cache)) {
        DRAW_WITH_ALPHA(p_gtk_paint_check ( style, pixmap, state, shadow, NULL, window, "checkbutton", 0, 0, rect.width(), rect.height()));
        if (usePixmapCache)
            QPixmapCache::insert(pixmapName, cache);
    }
    painter->drawPixmap(rect.topLeft(), cache);
}

QPalette QGtkStyle::standardPalette() const
{
    QPalette palette = QCleanlooksStyle::standardPalette();
    GtkStyle *style = getGtkStyle();

    if (style) {
        GdkColor gdkBg, gdkBase, gdkText, gdkSbg, gdkSfg;
        QColor bg, base, fg, highlight, highlightText;
        gdkBg = style->bg[GTK_STATE_NORMAL];
        gdkText = style->fg[GTK_STATE_NORMAL];
        gdkBase = style->base[GTK_STATE_NORMAL];

        // Our selected color is primarily used for text
        // so we assume a gtkEntry will have the most correct
        // value
        GtkWidget *gtkEntry = get_gtk_entry();
        gdkSbg = gtkEntry->style->base[GTK_STATE_SELECTED];
        gdkSfg = gtkEntry->style->text[GTK_STATE_SELECTED];

        bg = QColor(gdkBg.red>>8, gdkBg.green>>8, gdkBg.blue>>8);
        fg = QColor(gdkText.red>>8, gdkText.green>>8, gdkText.blue>>8);
        base = QColor(gdkBase.red>>8, gdkBase.green>>8, gdkBase.blue>>8);
        highlight = QColor(gdkSbg.red>>8, gdkSbg.green>>8, gdkSbg.blue>>8);
        highlightText = QColor(gdkSfg.red>>8, gdkSfg.green>>8, gdkSfg.blue>>8);

        palette.setColor(QPalette::HighlightedText, highlightText);
        palette.setColor(QPalette::Light, bg.lighter(125));
        palette.setColor(QPalette::Shadow, bg.darker(130));
        palette.setColor(QPalette::Dark, bg.darker(120));
        palette.setColor(QPalette::Text, fg);
        palette.setColor(QPalette::WindowText, fg);
        palette.setColor(QPalette::ButtonText, fg);
        palette.setColor(QPalette::Base, base);
        palette.setColor(QPalette::Window, bg);
        palette.setColor(QPalette::Button, bg);
        QColor disabled((fg.red()   + bg.red())  / 2,
                        (fg.green() + bg.green())/ 2,
                        (fg.blue()  + bg.blue()) / 2);
        palette.setColor(QPalette::Disabled, QPalette::Text, disabled);
        palette.setColor(QPalette::Disabled, QPalette::WindowText, disabled);
        palette.setColor(QPalette::Disabled, QPalette::Foreground, disabled);
        palette.setColor(QPalette::Disabled, QPalette::ButtonText, disabled);
        palette.setColor(QPalette::Highlight, highlight);
        // calculate disabled colors by removing saturation
        highlight.setHsv(highlight.hue(), 0, highlight.value(), highlight.alpha());
        highlightText.setHsv(highlightText.hue(), 0, highlightText.value(), highlightText.alpha());
        palette.setColor(QPalette::Disabled, QPalette::Highlight, highlight);
        palette.setColor(QPalette::Disabled, QPalette::HighlightedText, highlightText);
    }
    return palette;
}

void updateFont()
{
    if (qApp->desktopSettingsAware()) {
        QFont font = qApp->font();
        GtkStyle *style = getGtkStyle();
        PangoFontDescription *gtk_font = style->font_desc;

        font.setPointSizeF((float)(pango_font_description_get_size(gtk_font)/PANGO_SCALE));

        QString family(pango_font_description_get_family(gtk_font));
        if (!family.isEmpty())
            font.setFamily(family);

        int weight = pango_font_description_get_weight(gtk_font);
        if (weight >= PANGO_WEIGHT_HEAVY)
            font.setWeight(QFont::Black);
        else if (weight >= PANGO_WEIGHT_BOLD)
            font.setWeight(QFont::Bold);
        else if (weight >= PANGO_WEIGHT_SEMIBOLD)
            font.setWeight(QFont::DemiBold);
        else if (weight >= PANGO_WEIGHT_NORMAL)
            font.setWeight(QFont::Normal);
        else
            font.setWeight(QFont::Light);

        PangoStyle fontstyle = pango_font_description_get_style(gtk_font);
        if (fontstyle == PANGO_STYLE_ITALIC)
            font.setStyle(QFont::StyleItalic);
        else if (fontstyle == PANGO_STYLE_OBLIQUE)
            font.setStyle(QFont::StyleOblique);
        else
            font.setStyle(QFont::StyleNormal);
        qApp->setFont(font);
    }
}

void QGtkStyle::polish(QPalette &palette)
{
    QCleanlooksStyle::polish(palette);
}

void QGtkStyle::polish(QApplication *app)
{
    QCleanlooksStyle::polish(app);
#if (QT_VERSION < 0x040400)
    // Pixmap cache is too low to be usable in 4.3
    QPixmapCache::setCacheLimit(10*1024);    
#endif
    QPixmapCache::clear();

    oldApplicationFont = app->font();
    app->setPalette(standardPalette());
    updateFont();

    if (0) {
        // ### Kill the unused warning for now
        p_gtk_frame_new();
        get_gtk_frame();
    }        
}

void QGtkStyle::unpolish(QApplication *app)
{
    QCleanlooksStyle::unpolish(app);
    QPixmapCache::clear();
    if (qApp->desktopSettingsAware())
        app->setFont(oldApplicationFont);
}

void QGtkStyle::polish(QWidget *widget)
{
    QCleanlooksStyle::polish(widget);
    if (qobject_cast<QAbstractButton*>(widget)
            || qobject_cast<QToolButton*>(widget)
            || qobject_cast<QTabBar*>(widget)
            || qobject_cast<QComboBox*>(widget)
            || qobject_cast<QScrollBar*>(widget)
            || qobject_cast<QSlider*>(widget)
            || qobject_cast<QAbstractSpinBox*>(widget)
            || qobject_cast<QSpinBox*>(widget)
            || qobject_cast<QHeaderView*>(widget))
        widget->setAttribute(Qt::WA_Hover);
    else if (QTreeView *tree = qobject_cast<QTreeView *> (widget)) {
        tree->viewport()->setAttribute(Qt::WA_Hover);
    }
    else if (QMenu *menu = qobject_cast<QMenu *> (widget)) {        
        menu->setAttribute(Qt::WA_StyledBackground);
    }
}

void QGtkStyle::unpolish(QWidget *widget)
{
    QCleanlooksStyle::unpolish(widget);
}

int QGtkStyle::pixelMetric(PixelMetric metric,
                           const QStyleOption *option,
                           const QWidget *widget) const
{
    GtkStyle *style = getGtkStyle();
    if (!style)
        return QCleanlooksStyle::pixelMetric(metric, option, widget);

    switch (metric) {
    case PM_ButtonShiftHorizontal: {
        GtkWidget *gtkButton = get_gtk_button();
        guint horizontal_shift;
	    gtk_widget_style_get (gtkButton, "child-displacement-x", &horizontal_shift, NULL);
        return horizontal_shift;
    }
    case PM_ButtonShiftVertical: {
        GtkWidget *gtkButton = get_gtk_button();
        guint vertical_shift;
	    gtk_widget_style_get (gtkButton, "child-displacement-y", &vertical_shift, NULL);
        return vertical_shift;
    }    
    case PM_MenuBarPanelWidth:
        return 0;
    case PM_MenuPanelWidth: {
            GtkWidget *gtkMenu = get_gtk_menu();
            guint horizontal_padding;
            // horizontal-padding is used by Maemo to get thicker borders
	        gtk_widget_style_get (gtkMenu, "horizontal-padding", &horizontal_padding, NULL);
            int padding = qMax<int>(gtkMenu->style->xthickness, horizontal_padding);
            return padding;
        }
    case PM_TitleBarHeight:
        return 22;
    case PM_MenuVMargin:
    case PM_MenuHMargin:
        return 0;
    case PM_DockWidgetTitleMargin:
        return 4;
    case PM_DockWidgetTitleBarButtonMargin:
        return 4;
    case PM_TabBarTabVSpace:
        return 12;
    case PM_TabBarTabHSpace:
        return 14;
    case PM_TabBarTabShiftVertical:
        return 3;
    case PM_ToolBarHandleExtent:
        return 9;
    case PM_SplitterWidth:
        return 6;
    case PM_SliderThickness:
    case PM_SliderControlThickness: {
            gint val;
            p_gtk_widget_style_get (get_gtk_hscale(), "slider-width", &val, NULL);
            return val;
        }
    case PM_ScrollBarExtent: {
            gint sliderLength;
            gint trough_border;
            GtkWidget *hScrollbar = get_gtk_hscrollbar();
 
           Q_ASSERT(p_gtk_widget_style_get);

            p_gtk_widget_style_get (hScrollbar,
                                  "trough-border",   &trough_border,
                                  "slider-width",    &sliderLength, 
                                  NULL);
            return sliderLength + trough_border*2;
        }
    case PM_ScrollBarSliderMin:
        return 34;
    case PM_SliderLength:
        gint val;
        p_gtk_widget_style_get (get_gtk_hscale(), "slider-length", &val, NULL);
        return val;
    case PM_ExclusiveIndicatorWidth:
    case PM_ExclusiveIndicatorHeight:
    case PM_IndicatorWidth:
    case PM_IndicatorHeight:
        {
            gint val;
            p_gtk_widget_style_get (get_gtk_check_button(), "indicator-size", &val, NULL);
            return val;
        }
    case PM_MenuBarVMargin: {
        GtkWidget *gtkMenubar = get_gtk_menubar();
        return  qMax(0, gtkMenubar->style->ythickness);
    }
    default:
        return QCleanlooksStyle::pixelMetric(metric, option, widget);
    }
}

int QGtkStyle::styleHint(StyleHint hint, const QStyleOption *option, const QWidget *widget,
                         QStyleHintReturn *returnData = 0) const
{
    GtkStyle *style = getGtkStyle();
    if (!style)
        return QCleanlooksStyle::styleHint(hint, option, widget, returnData);

    switch (hint) {
    case SH_SpinControls_DisableOnBounds:
        return true;
    case SH_DitherDisabledText:
        return int(false);
    case SH_EtchDisabledText:
        return int(true);
    case SH_Menu_SubMenuPopupDelay:
        return 225; // Default from gtkmenu for now
    default:
        return QCleanlooksStyle::styleHint(hint, option, widget, returnData);
    }
}

void QGtkStyle::drawPrimitive(PrimitiveElement element,
                              const QStyleOption *option,
                              QPainter *painter,
                              const QWidget *widget) const
{
    GtkStyle* style = getGtkStyle();
    GtkWidget *gtkWindow = get_gtk_window();

    if (!style) {
        QCleanlooksStyle::drawPrimitive(element, option, painter, widget);
        return;
    }
    switch (element) {
    case PE_PanelTipLabel: {
            style = gtk_rc_get_style_by_paths(p_gtk_settings_get_default(), "gtk-tooltips", "GtkWindow", GTK_TYPE_WINDOW);
            qgtkstyle_paint_flat_box(gtkWindow, NULL, "tooltip", option->rect, GTK_STATE_NORMAL, GTK_SHADOW_NONE, style, painter);
        }
        break;
    case PE_IndicatorHeaderArrow:
        if (const QStyleOptionHeader *header = qstyleoption_cast<const QStyleOptionHeader *>(option)) {
            GtkWidget *gtkTreeHeader = get_gtk_tree_header();
            GtkStateType state = gtkState(option);
            style = gtkTreeHeader->style;
            GtkArrowType type = GTK_ARROW_UP;
            QRect r = header->rect;
            QImage arrow;
            if (header->sortIndicator & QStyleOptionHeader::SortUp)
                type = GTK_ARROW_UP;
            else if (header->sortIndicator & QStyleOptionHeader::SortDown)
                type = GTK_ARROW_DOWN;
            qgtkstyle_paint_arrow(gtkWindow, gtkTreeHeader, "button", option->rect.adjusted(1, 1, -1, -1), type, state, 
                                    GTK_SHADOW_NONE, FALSE, style, painter);
        }
        break;
    case PE_FrameFocusRect:
        if (qobject_cast<const QAbstractItemView*>(widget))
            return;
        else {
            // ### this mess should move to subcontrolrect
            QRect frameRect = option->rect.adjusted(1, 1, -1, -2);
            if (qobject_cast<const QTabBar*>(widget))
                frameRect.adjust(-1, 1, 1, 1);
            qgtkstyle_paint_focus(gtkWindow, NULL, "tab", frameRect, GTK_STATE_ACTIVE, style, painter);
        }        
        break;
    case PE_IndicatorBranch:
        if (option->state & State_Children) {
            QRect rect = option->rect;
            rect = QRect(0, 0, 10, 10);
            rect.moveCenter(option->rect.center());
            rect.translate(2, 0);
            GtkExpanderStyle openState = GTK_EXPANDER_EXPANDED;
            GtkExpanderStyle closedState = GTK_EXPANDER_COLLAPSED;
               qgtkstyle_paint_expander(gtkWindow, NULL, "expander", rect, (option->state & State_MouseOver) ? GTK_STATE_PRELIGHT : GTK_STATE_NORMAL, 
            option->state & State_Open ? openState : closedState , style, painter);
        } //else
            //QCleanlooksStyle::drawPrimitive(element, option, painter, widget);
//        qgtkstyle_paint_expander(gtkWindow, NULL, "expander", option->rect, GTK_STATE_NORMAL, GTK_EXPANDER_COLLAPSED , style, painter);
//        qgtkstyle_paint_flat_box(gtkWindow, NULL, "expander", option->rect, GTK_STATE_NORMAL, GTK_SHADOW_NONE, style, painter);

        break;
#if (QT_VERSION >= 0x040400)
    case PE_PanelItemViewItem:
        if (const QStyleOptionViewItemV4 *vopt = qstyleoption_cast<const QStyleOptionViewItemV4 *>(option)) {
            GtkWidget *gtkTreeView = get_gtk_treeview();
            if (vopt->state & State_Selected) {
                QLinearGradient gradient;
                gradient.setStart(option->rect.left(), option->rect.top());
                gradient.setFinalStop(option->rect.left(), option->rect.bottom());
                gradient.setColorAt(0, option->palette.highlight().color().lighter(105));
                ;
                gradient.setColorAt(0.5, option->palette.highlight().color().lighter(101));
                gradient.setColorAt(0.51, option->palette.highlight().color().darker(101));
                gradient.setColorAt(1, option->palette.highlight().color().darker(105));
                ;
                painter->fillRect(option->rect, gradient);
                //qgtkstyle_paint_flat_box(gtkWindow, gtkTreeView, "treeview", option->rect, GTK_STATE_SELECTED, GTK_SHADOW_NONE, style, painter);
            } else if (vopt->features & QStyleOptionViewItemV2::Alternate)
                qgtkstyle_paint_flat_box(gtkWindow, gtkTreeView, "cell-odd", option->rect, GTK_STATE_NORMAL, GTK_SHADOW_NONE, style, painter);
        }
        break;
#endif
    case PE_IndicatorToolBarHandle: {
            GtkWidget *gtkToolbar = get_gtk_toolbar();
            GtkShadowType shadow_type;  
            gtk_widget_style_get (gtkToolbar, "shadow-type", &shadow_type, NULL);
            //Note when the toolbar is horizontal, the handle is vertical
            painter->setClipRect(option->rect);
            qgtkstyle_paint_handle(gtkWindow, gtkToolbar, "toolbar", option->rect.adjusted(-1, -1 ,0 ,1),
                                     GTK_STATE_NORMAL, shadow_type, !(option->state & State_Horizontal) ?
                                     GTK_ORIENTATION_HORIZONTAL : GTK_ORIENTATION_VERTICAL, gtkToolbar->style, painter);
        }
        break;
    case PE_IndicatorArrowUp:
    case PE_IndicatorArrowDown:
    case PE_IndicatorArrowLeft:
    case PE_IndicatorArrowRight: {
            GtkArrowType type = GTK_ARROW_UP;
            switch (element) {
            case PE_IndicatorArrowDown:
                type = GTK_ARROW_DOWN;
                break;
            case PE_IndicatorArrowLeft:
                type = GTK_ARROW_LEFT;
                break;
            case PE_IndicatorArrowRight:
                type = GTK_ARROW_RIGHT;
                break;
            default:
                break;
            }
            GtkShadowType shadow = option->state & State_Sunken ? GTK_SHADOW_IN : GTK_SHADOW_OUT;
            GtkStateType state = gtkState(option);
            qgtkstyle_paint_arrow(gtkWindow, get_gtk_vscrollbar(), "vscrollbar", option->rect, type, state, shadow, FALSE, style, painter);
        }
        break;
#ifndef QT_NO_GROUPBOX
    case PE_FrameGroupBox:
        // Do nothing here, the GNOME groupboxes are flat
        break;
#endif // QT_NO_GROUPBOX
/*    case PE_Frame:
         {
            get_gtk_frame();
            GtkShadowType shadow_type;
            g_object_get(gtkFrame, "shadow-type", &shadow_type, NULL);
            qgtkstyle_paint_box(gtkWindow, gtkMenubar, "toolbar",  option->rect,
                                  GTK_STATE_NORMAL, GTK_SHADOW_NONE, gtkFrame->style, painter);
        }
        break;
*/
    case PE_FrameMenu:
        //This is actually done by PE_Widget due to a clipping issue
        //Otherwise Menu items will not be able to span the entire menu width

        // This is only used by floating tool bars
        if (qobject_cast<const QToolBar *>(widget)) {
            GtkWidget *gtkMenubar = get_gtk_menubar();
            qgtkstyle_paint_box(gtkWindow, gtkMenubar, "toolbar",  option->rect,
                                  GTK_STATE_NORMAL, GTK_SHADOW_OUT, style, painter);
            qgtkstyle_paint_box(gtkWindow, gtkMenubar, "menu",  option->rect,
                                  GTK_STATE_NORMAL, GTK_SHADOW_OUT, style, painter);
        }
        break;

    case PE_FrameLineEdit: {
            GtkWidget *gtkEntry = get_gtk_entry();
            GtkStateType state = gtkState(option);
            if (option->state & State_HasFocus)
                GTK_WIDGET_SET_FLAGS(gtkEntry, GTK_HAS_FOCUS);
            else
                GTK_WIDGET_UNSET_FLAGS(gtkEntry, GTK_HAS_FOCUS);
            qgtkstyle_paint_shadow(gtkWindow, gtkEntry, "entry", option->rect, state, GTK_SHADOW_IN, gtkEntry->style,
                                     painter, option->state & State_HasFocus ? "focus" : QString());
            int xt = gtkEntry->style->xthickness;
            int yt = gtkEntry->style->ythickness;
            painter->fillRect(option->rect.adjusted(xt, yt, -xt, -yt), option->palette.base().color());
        }
        break;

    case PE_FrameTabWidget:
        if (const QStyleOptionTabWidgetFrame *frame = qstyleoption_cast<const QStyleOptionTabWidgetFrame*>(option)) {
            GtkWidget *gtkNotebook = get_gtk_notebook();
            style = get_style(gtkNotebook);
            GtkShadowType shadow = GTK_SHADOW_OUT;
            GtkStateType state = gtkState(option);
            if (const QTabWidget *tabwidget = qobject_cast<const QTabWidget*>(widget)) {
                QTabBar *tabBar = tabwidget->findChild<QTabBar*>();
                if (tabwidget->count() > 0 ) {
                    QRect tabRect = tabBar->tabRect(tabBar->currentIndex());
                    int begin = frame->leftCornerWidgetSize.width() + tabRect.left();
                    int size = tabRect.width();
                    // No gap if we do not show the actual tabs
                    if (!tabBar->isVisible())
                        qgtkstyle_paint_box(gtkWindow, gtkNotebook, "notebook",  option->rect, state, shadow, style, painter);
                    // We do not currently cache this huge area
                    else if (frame->shape == QTabBar::RoundedNorth)
                        qgtkstyle_paint_box_gap(gtkWindow, gtkNotebook, "notebook",  option->rect, state, shadow, GTK_POS_TOP,
                                                  begin, size, style, painter);
                    else if (frame->shape == QTabBar::RoundedSouth)
                        qgtkstyle_paint_box_gap(gtkWindow, gtkNotebook, "notebook",  option->rect, state, shadow, GTK_POS_BOTTOM,
                                                  begin, size, style, painter);
                    else if (frame->shape == QTabBar::RoundedWest)
                        qgtkstyle_paint_box_gap(gtkWindow, gtkNotebook, "notebook",  option->rect, state, shadow, GTK_POS_LEFT,
                                                  begin, size, style, painter);
                    else
                        qgtkstyle_paint_box_gap(gtkWindow, gtkNotebook, "notebook",  option->rect, state, shadow, GTK_POS_RIGHT,
                                                  begin, size, style, painter);
                } else {
                    qgtkstyle_paint_box(gtkWindow, gtkNotebook, "notebook",  option->rect, state, shadow, style, painter);

                }
            }
            else
                qgtkstyle_paint_box(gtkWindow, gtkNotebook, "notebook",  option->rect, state, shadow, style, painter);
        }
        break;
    case PE_PanelButtonCommand: {
            GtkStateType state = gtkState(option);
            if (option->state & State_On || option->state & State_Sunken)
                state = GTK_STATE_ACTIVE;

            GtkWidget *gtkButton = get_gtk_button();
            style = gtkButton->style;

            gtkButton->allocation.x = -2;
            gtkButton->allocation.y = -1;

            bool isDefault = false;
            if (const QStyleOptionButton *btn = qstyleoption_cast<const QStyleOptionButton*>(option))
                isDefault = btn->features & QStyleOptionButton::DefaultButton;

            if (isDefault)
                GTK_WIDGET_SET_FLAGS(gtkButton, GTK_HAS_DEFAULT);
            else
                GTK_WIDGET_UNSET_FLAGS(gtkButton, GTK_HAS_DEFAULT);

            GtkShadowType shadow = (option->state & State_Sunken || option->state & State_On ) ?
                                   GTK_SHADOW_IN : GTK_SHADOW_OUT;
            QRect buttonRect = option->rect;
            qgtkstyle_paint_box(gtkWindow, gtkButton, "button", buttonRect, state, shadow, style, painter, isDefault ? "d" : QString());
        }
        break;
    case PE_IndicatorRadioButton: {
            GtkShadowType shadow = GTK_SHADOW_OUT;
            GtkStateType state = gtkState(option);
            if (option->state & State_Sunken)
                state = GTK_STATE_ACTIVE;
            if (option->state & State_NoChange)
                shadow = GTK_SHADOW_ETCHED_IN;
            else if (option->state & State_On)
                shadow = GTK_SHADOW_IN;
            else
                shadow = GTK_SHADOW_OUT;

            qgtkstyle_paint_option(gtkWindow, option->rect, state, shadow, style, painter);

        }
        break;

    case PE_IndicatorCheckBox: {
            GtkWidget *gtkCheckButton = get_gtk_check_button();
            GtkShadowType shadow = GTK_SHADOW_OUT;
            GtkStateType state = gtkState(option);
            if (option->state & State_Sunken)
                state = GTK_STATE_ACTIVE;

            if (option->state & State_NoChange)
                shadow = GTK_SHADOW_ETCHED_IN;
            else if (option->state & State_On)
                shadow = GTK_SHADOW_IN;
            else
                shadow = GTK_SHADOW_OUT;
            qgtkstyle_paint_checkbox(gtkCheckButton, option->rect, state, shadow, gtkCheckButton->style, painter);
        }
        break;
#ifndef QT_NO_TABBAR
    case PE_FrameTabBarBase:
        if (const QStyleOptionTabBarBase *tbb
                = qstyleoption_cast<const QStyleOptionTabBarBase *>(option)) {
            QRect tabRect = tbb->rect;
            QRegion region(tabRect);
            painter->save();
            painter->setPen(QPen(option->palette.shadow().color().dark(110), 0));
            switch (tbb->shape) {
            case QTabBar::RoundedNorth:
                region -= tbb->selectedTabRect.adjusted(1, 0, -1, 0);;
                painter->setClipRegion(region);
                painter->drawLine(tabRect.topLeft(), tabRect.topRight());
                break;
            case QTabBar::RoundedWest:
                region -= tbb->selectedTabRect;
                painter->setClipRegion(region);
                painter->drawLine(tabRect.left(), tabRect.top(), tabRect.left(), tabRect.bottom());
                break;
            case QTabBar::RoundedSouth:
                region -= tbb->selectedTabRect.adjusted(1, 0, -1, 0);;
                painter->setClipRegion(region);
                painter->drawLine(tbb->rect.left(), tbb->rect.bottom(),
                            tabRect.right(), tabRect.bottom());
                break;
            case QTabBar::RoundedEast:
                region -= tbb->selectedTabRect;
                painter->setClipRegion(region);
                painter->drawLine(tabRect.topRight(), tabRect.bottomRight());
                break;
            case QTabBar::TriangularNorth:
            case QTabBar::TriangularEast:
            case QTabBar::TriangularWest:
            case QTabBar::TriangularSouth:
                painter->restore();
                QWindowsStyle::drawPrimitive(element, option, painter, widget);
                return;
            }
            painter->restore();
        }
        return;
#endif // QT_NO_TABBAR

    case PE_Widget:
        if (qobject_cast<const QMenu *>(widget)) {
            GtkWidget *gtkMenu = get_gtk_menu();
            GdkColor gdkBg = gtkMenu->style->bg[0];
            QColor menuBackground = QColor(gdkBg.red>>8, gdkBg.green>>8, gdkBg.blue>>8);
            GtkStateType state = GTK_STATE_NORMAL;
            GtkShadowType shadow = GTK_SHADOW_OUT;
            painter->fillRect(option->rect, menuBackground);
            // Note, ALPHA disabled for performance reasons
            qgtkstyle_paint_box(gtkWindow, gtkMenu, "menu", option->rect, state, shadow, gtkMenu->style, painter, false);
        }/* else if (const QMainWindow *mw = qobject_cast<const QMainWindow *>(widget)) { // For status bar line
            if (QStatusBar *statusBar = mw->findChild<QStatusBar*>()) {
                QRect windowRect = statusBar->geometry();
                painter->setPen(option->palette.dark().color());
                painter->drawLine(windowRect.topLeft(), windowRect.topRight());
            }
        }*/
        break;


    default:
        QCleanlooksStyle::drawPrimitive(element, option, painter, widget);
    }
}



void QGtkStyle::drawComplexControl(ComplexControl control, const QStyleOptionComplex *option,
                                   QPainter *painter, const QWidget *widget) const
{
    GtkWidget *gtkWindow = get_gtk_window();
    GtkStyle* style = getGtkStyle();

    if (!style) {
        QCleanlooksStyle::drawComplexControl(control, option, painter, widget);
        return;
    }

    QColor button = option->palette.button().color();
    QColor dark;
    QColor grooveColor;
    QColor darkOutline;
    dark.setHsv(button.hue(),
                CL_MIN(255, (int)(button.saturation()*1.9)),
                CL_MIN(255, (int)(button.value()*0.7)));
    grooveColor.setHsv(button.hue(),
                       CL_MIN(255, (int)(button.saturation()*2.6)),
                       CL_MIN(255, (int)(button.value()*0.9)));
    darkOutline.setHsv(button.hue(),
                       CL_MIN(255, (int)(button.saturation()*3.0)),
                       CL_MIN(255, (int)(button.value()*0.6)));

    QColor alphaCornerColor;
    if (widget) {
        // ### backgroundrole/foregroundrole should be part of the style option
        alphaCornerColor = mergedColors(option->palette.color(widget->backgroundRole()), darkOutline);
    } else {
        alphaCornerColor = mergedColors(option->palette.background().color(), darkOutline);
    }
    QPalette palette = option->palette;

    switch (control) {
    case CC_TitleBar:
        if (const QStyleOptionTitleBar *tb = qstyleoption_cast<const QStyleOptionTitleBar *>(option)) {       
            // Since this is drawn by metacity and not Gtk we 
            // have to rely on Cleanlooks for a fallback
            QStyleOptionTitleBar copyOpt = *tb;
            QPalette pal = copyOpt.palette;
            // Bg color is closer to the window selection than
            // the base selection color
            GdkColor gdkBg = style->bg[GTK_STATE_SELECTED];
            QColor bgColor(gdkBg.red>>8, gdkBg.green>>8, gdkBg.blue>>8);
            pal.setBrush(QPalette::Active, QPalette::Highlight, bgColor);
            copyOpt.palette = pal;
            QCleanlooksStyle::drawComplexControl(control, &copyOpt, painter, widget);
        }
        break;
#ifndef QT_NO_GROUPBOX
    case CC_GroupBox:
        painter->save();
        if (const QStyleOptionGroupBox *groupBox = qstyleoption_cast<const QStyleOptionGroupBox *>(option)) {
            QRect textRect = subControlRect(CC_GroupBox, groupBox, SC_GroupBoxLabel, widget);
            QRect checkBoxRect = subControlRect(CC_GroupBox, groupBox, SC_GroupBoxCheckBox, widget);
            // Draw title
            if ((groupBox->subControls & QStyle::SC_GroupBoxLabel) && !groupBox->text.isEmpty()) {
                if (!groupBox->text.isEmpty()) {
                    QColor textColor = groupBox->textColor;
                    if (textColor.isValid())
                        painter->setPen(textColor);
                    int alignment = int(groupBox->textAlignment);
                    if (!styleHint(QStyle::SH_UnderlineShortcut, option, widget))
                        alignment |= Qt::TextHideMnemonic;
                    QFont font = painter->font();
                    font.setBold(true);
                    painter->setFont(font);
                    painter->drawText(textRect, Qt::TextShowMnemonic | Qt::AlignLeft| alignment, groupBox->text);
                    if (option->state & State_HasFocus)
                        qgtkstyle_paint_focus(gtkWindow, NULL, "tab", textRect.adjusted(-4, -1, 0, -3), GTK_STATE_ACTIVE, style, painter);
                }
            }
            if (groupBox->subControls & SC_GroupBoxCheckBox) {
                QStyleOptionButton box;
                box.QStyleOption::operator=(*groupBox);
                box.rect = checkBoxRect;
                drawPrimitive(PE_IndicatorCheckBox, &box, painter, widget);
            }
        }
        painter->restore();
        break;
#endif // QT_NO_GROUPBOX

#ifndef QT_NO_COMBOBOX
    case CC_ComboBox:
        painter->save();
        if (const QStyleOptionComboBox *comboBox = qstyleoption_cast<const QStyleOptionComboBox *>(option)) {
            GtkWidget *gtkButton = get_gtk_button();
            GtkWidget *gtkComboBox = get_gtk_combobox();
            GtkWidget *gtkComboBoxEntry = get_gtk_entry(); // get_gtk_combobox_entry does not work in some styles
            GtkWidget *gtkComboBoxEntryButton = get_gtk_combobox_entry_button();
            bool sunken = comboBox->state & State_On; // play dead, if combobox has no items
            bool isEnabled = (comboBox->state & State_Enabled);
            bool focus = isEnabled && (comboBox->state & State_HasFocus);
            QColor buttonShadow = option->palette.dark().color();
            GtkStateType state = gtkState(option);
            get_gtk_combobox();
            QPixmap cache;
            QString pixmapName;
            QStyleOptionComboBox comboBoxCopy = *comboBox;
            comboBoxCopy.rect = option->rect;

            QRect rect = option->rect;
            QRect downArrowRect = subControlRect(CC_ComboBox, &comboBoxCopy,
                                                 SC_ComboBoxArrow, widget);
            QRect editRect = subControlRect(CC_ComboBox, &comboBoxCopy,
                                            SC_ComboBoxEditField, widget);
            // Draw a push button
            if (comboBox->editable) {
                QStyleOptionButton  buttonOption;
                buttonOption.QStyleOption::operator=(*comboBox);
                buttonOption.rect = rect;
                buttonOption.state = comboBox->state & (State_Enabled | State_MouseOver);

                buttonOption.state |= State_HasFocus;
                if (sunken) {
                    buttonOption.state |= State_Sunken;
                    buttonOption.state &= ~State_MouseOver;
                }

                // Draw line edit
                if (state == GTK_STATE_PRELIGHT)
                    state = GTK_STATE_NORMAL;
                get_gtk_combobox_entry();
                QRect entryRect = option->rect;
                entryRect.setRight(downArrowRect.left() + 2);

                qgtkstyle_paint_shadow(gtkWindow, gtkComboBoxEntry, "entry", entryRect, state, GTK_SHADOW_IN, 
                                       get_style(gtkComboBoxEntry), painter, option->state & State_HasFocus ? "f" : QString());

                GtkStateType state = gtkState(option);
                if (option->state & State_On || option->state & State_Sunken)
                    state = GTK_STATE_ACTIVE;

                gtkComboBoxEntryButton->allocation.x = -2;
                gtkComboBoxEntryButton->allocation.y = -1;

                bool isDefault = false;
                if (const QStyleOptionButton *btn = qstyleoption_cast<const QStyleOptionButton*>(option))
                    isDefault = btn->features & QStyleOptionButton::DefaultButton;

                if (isDefault)
                    GTK_WIDGET_SET_FLAGS(gtkButton, GTK_HAS_DEFAULT);
                else
                    GTK_WIDGET_UNSET_FLAGS(gtkButton, GTK_HAS_DEFAULT);

                GtkShadowType shadow = (option->state & State_Sunken || option->state & State_On ) ?
                                       GTK_SHADOW_IN : GTK_SHADOW_OUT;
                QRect buttonRect = option->rect;
                buttonRect.setLeft(downArrowRect.left() - 1);
                qgtkstyle_paint_box(gtkWindow, gtkComboBoxEntryButton, "button", buttonRect, 
                                    state, shadow, gtkComboBoxEntryButton->style, painter, isDefault ? "d" : QString());

            } else {
                QStyleOptionButton buttonOption;
                buttonOption.QStyleOption::operator=(*comboBox);
                buttonOption.rect = rect;
                buttonOption.state = comboBox->state & (State_Enabled | State_MouseOver);
                if (sunken) {
                    buttonOption.state |= State_Sunken;
                    buttonOption.state &= ~State_MouseOver;
                }
                if (option->state & State_HasFocus) {
                    buttonOption.features |= QStyleOptionButton::DefaultButton;
                }
                drawPrimitive(PE_PanelButtonCommand, &buttonOption, painter, widget);
                painter->setPen(buttonShadow.darker(102));
                int borderSize = 4;
                if (!sunken) {
                    if (comboBox->direction == Qt::RightToLeft) {
                        painter->drawLine(QPoint(downArrowRect.right() + 1, downArrowRect.top() + borderSize),
                                          QPoint(downArrowRect.right() + 1, downArrowRect.bottom() - borderSize));
                        painter->setPen(option->palette.light().color());
                        painter->drawLine(QPoint(downArrowRect.right(), downArrowRect.top() + borderSize),
                                          QPoint(downArrowRect.right(), downArrowRect.bottom() - borderSize));
                    } else {
                        painter->drawLine(QPoint(downArrowRect.left() - 1, downArrowRect.top() + borderSize),
                                          QPoint(downArrowRect.left() - 1, downArrowRect.bottom() - borderSize));
                        painter->setPen(option->palette.light().color());
                        painter->drawLine(QPoint(downArrowRect.left() , downArrowRect.top() + borderSize),
                                          QPoint(downArrowRect.left() , downArrowRect.bottom() - borderSize));
                    }
                } else {
                    painter->setPen(dark.lighter(110));
                    if (comboBox->direction == Qt::RightToLeft) {
                        painter->drawLine(QPoint(downArrowRect.right() + 1, downArrowRect.top() + borderSize),
                                          QPoint(downArrowRect.right() + 1, downArrowRect.bottom() - borderSize));
                    } else {
                        painter->drawLine(QPoint(downArrowRect.left() - 1, downArrowRect.top() + borderSize),
                                          QPoint(downArrowRect.left() - 1, downArrowRect.bottom() - borderSize));
                    }
                }
            }

            if (comboBox->subControls & SC_ComboBoxArrow) {
                if (comboBox->editable) {
                    // Draw the down arrow
                    int margin = 5;
                    int xoff = option->state & State_Sunken ? pixelMetric(PM_ButtonShiftHorizontal, option, widget) : 0;
                    int yoff = option->state & State_Sunken ? pixelMetric(PM_ButtonShiftVertical, option, widget) : 0;
                    qgtkstyle_paint_arrow(gtkWindow, gtkComboBoxEntry, "combo",
                                          downArrowRect.adjusted(margin + xoff, margin + yoff, -margin + xoff, -margin + yoff), 
                                          GTK_ARROW_DOWN, state, GTK_SHADOW_NONE, FALSE, gtkComboBoxEntry->style, painter);
                } else {
                    int margin = 5;
                    qgtkstyle_paint_arrow(gtkWindow, gtkComboBox, "combo",
                                          downArrowRect.adjusted(margin, margin, -margin, -margin), 
                                          GTK_ARROW_DOWN, state, GTK_SHADOW_NONE, FALSE, gtkComboBoxEntry->style, painter);
            }
            // Draw the focus rect
                if ((focus && (option->state & State_KeyboardFocusChange)) && !comboBox->editable) {
                    qgtkstyle_paint_focus(gtkWindow, NULL, "button", option->rect.adjusted(2, 2, -2, -2), GTK_STATE_ACTIVE, style, painter);
                }
            }
        }
        painter->restore();
        break;
#endif // QT_NO_COMBOBOX
#ifndef QT_NO_TOOLBUTTON
    case CC_ToolButton:
        if (const QStyleOptionToolButton *toolbutton
                = qstyleoption_cast<const QStyleOptionToolButton *>(option)) {
            QRect button, menuarea;
            button = subControlRect(control, toolbutton, SC_ToolButton, widget);
            menuarea = subControlRect(control, toolbutton, SC_ToolButtonMenu, widget);
            State bflags = toolbutton->state;
            if (bflags & State_AutoRaise)
                if (!(bflags & State_MouseOver))
                    bflags &= ~State_Raised;
            State mflags = bflags;
            if (toolbutton->activeSubControls & SC_ToolButton)
                bflags |= State_Sunken;
            if (toolbutton->activeSubControls & SC_ToolButtonMenu)
                mflags |= State_Sunken;
            QStyleOption tool(0);
            tool.palette = toolbutton->palette;
            if (toolbutton->subControls & SC_ToolButton) {
                if (bflags & (State_Sunken | State_On | State_Raised)) {
                    tool.rect = button;
                    tool.state = bflags;
                    drawPrimitive(PE_PanelButtonTool, &tool, painter, widget);
                }
            }
            if (toolbutton->state & State_HasFocus) {
                QStyleOptionFocusRect fr;
                fr.QStyleOption::operator=(*toolbutton);
                fr.rect.adjust(3, 3, -3, -3);
                if (toolbutton->features & QStyleOptionToolButton::MenuButtonPopup)
                    fr.rect.adjust(0, 0, -pixelMetric(QStyle::PM_MenuButtonIndicator, toolbutton, widget), 0);
                drawPrimitive(PE_FrameFocusRect, &fr, painter, widget);
            }
            QStyleOptionToolButton label = *toolbutton;
            int fw = pixelMetric(PM_DefaultFrameWidth, option, widget);
            label.rect = button.adjusted(fw, fw, -fw, -fw);
            drawControl(CE_ToolButtonLabel, &label, painter, widget);
            if (toolbutton->subControls & SC_ToolButtonMenu) {
                int margin = 7;
                tool.rect = menuarea;
                tool.state = mflags;
                if (mflags & State_Enabled && mflags & (State_Sunken | State_Raised))
                    drawPrimitive(PE_IndicatorButtonDropDown, &tool, painter, widget);
                tool.rect.adjust(margin, margin, -margin, -margin);
                drawPrimitive(PE_IndicatorArrowDown, &tool, painter, widget);
            } else if (toolbutton->features & QStyleOptionToolButton::HasMenu) {
                int mbi = pixelMetric(PM_MenuButtonIndicator, toolbutton, widget);
                QRect ir = toolbutton->rect;
                QStyleOptionToolButton newBtn = *toolbutton;
                newBtn.rect = QRect(ir.right() + 5 - mbi, ir.height() - mbi + 4, mbi - 6, mbi - 6);
                drawPrimitive(PE_IndicatorArrowDown, &newBtn, painter, widget);
            }
        }
        break;
#endif // QT_NO_TOOLBUTTON
#ifndef QT_NO_SCROLLBAR
    case CC_ScrollBar:
        if (const QStyleOptionSlider *scrollBar = qstyleoption_cast<const QStyleOptionSlider *>(option)) {
            GtkWidget *gtkHScrollBar = get_gtk_hscrollbar();
            GtkWidget *gtkVScrollBar = get_gtk_vscrollbar();

            // Fill background in case the scrollbar is partially transparent
            painter->fillRect(option->rect, option->palette.background());

            QRect rect = scrollBar->rect;
            QRect scrollBarSubLine = subControlRect(control, scrollBar, SC_ScrollBarSubLine, widget);
            QRect scrollBarAddLine = subControlRect(control, scrollBar, SC_ScrollBarAddLine, widget);
            QRect scrollBarSlider = subControlRect(control, scrollBar, SC_ScrollBarSlider, widget);
            QRect grooveRect = subControlRect(control, scrollBar, SC_ScrollBarGroove, widget);
            bool horizontal = scrollBar->orientation == Qt::Horizontal;
            GtkWidget * scrollbarWidget = horizontal ? gtkHScrollBar : gtkVScrollBar;
            style = scrollbarWidget->style;
            gboolean trough_under_steppers;
            gboolean trough_side_details;
            gboolean stepper_size;
            gint trough_border;
            p_gtk_widget_style_get ((GtkWidget*)(scrollbarWidget),
                                  "trough-border",   &trough_border,
                                  "trough-side-details",   &trough_side_details,
                                  "trough-under-steppers", &trough_under_steppers,
                                  "stepper-size",          &stepper_size, NULL);

            if (trough_under_steppers) {
                scrollBarAddLine.adjust(trough_border, trough_border, -trough_border, -trough_border);
                scrollBarSubLine.adjust(trough_border, trough_border, -trough_border, -trough_border);
                scrollBarSlider.adjust(horizontal ? -trough_border : 0, horizontal ? 0 : -trough_border, 
                                       horizontal ? trough_border : 0, horizontal ? 0 : trough_border);
            }
            int maximum = 2;
            int fakePos = 0;
            if (scrollBar->minimum == scrollBar->maximum)
                maximum = 0;
            if (scrollBar->sliderPosition == scrollBar->maximum)
                fakePos = maximum;
            else if (scrollBar->sliderPosition > scrollBar->minimum)
                fakePos = maximum - 1;
            GtkObject *adjustment =  p_gtk_adjustment_new(fakePos, 0, maximum, 0, 0, 0);

            if (horizontal)
                p_gtk_range_set_adjustment((GtkRange*)(gtkHScrollBar), (GtkAdjustment*)(adjustment));
            else
                p_gtk_range_set_adjustment((GtkRange*)(gtkVScrollBar), (GtkAdjustment*)(adjustment));

            if (scrollBar->subControls & SC_ScrollBarGroove) {
                GtkStateType state = GTK_STATE_ACTIVE;
                if (!(option->state & State_Enabled))
                    state = GTK_STATE_INSENSITIVE;

                if (trough_under_steppers)
                    grooveRect = option->rect;
                qgtkstyle_paint_box(gtkWindow, scrollbarWidget, "trough", grooveRect, state, GTK_SHADOW_IN, style, painter);
            }
            //paint slider
            if (scrollBar->subControls & SC_ScrollBarSlider) {
                GtkStateType state = GTK_STATE_NORMAL;
                if (!(option->state & State_Enabled))
                    state = GTK_STATE_INSENSITIVE;
                else if (option->state & State_MouseOver && (scrollBar->activeSubControls & SC_ScrollBarSlider))
                    state = GTK_STATE_PRELIGHT;
                GtkShadowType shadow = GTK_SHADOW_OUT;
                if (trough_under_steppers) {
                    if (!horizontal)
                        scrollBarSlider.adjust(trough_border, 0, -trough_border, 0);
                    else
                        scrollBarSlider.adjust(0, trough_border, 0, -trough_border);
                }
                qgtkstyle_paint_slider(gtkWindow, scrollbarWidget, "slider", scrollBarSlider, state, shadow, style, painter,
                                         horizontal ? GTK_ORIENTATION_HORIZONTAL : GTK_ORIENTATION_VERTICAL, QString("%0%1").arg(fakePos).arg(maximum));
            }

            if (scrollBar->subControls & SC_ScrollBarAddLine) {
                gtkVScrollBar->allocation.y = scrollBarAddLine.top();
                gtkVScrollBar->allocation.height = scrollBarAddLine.height() - rect.height() + 6; //###
                gtkHScrollBar->allocation.x = scrollBarAddLine.right();
                gtkHScrollBar->allocation.width = scrollBarAddLine.width() - rect.width();

                GtkShadowType shadow = GTK_SHADOW_OUT;
                GtkStateType state = GTK_STATE_NORMAL;
                if (!(option->state & State_Enabled) || (fakePos == maximum))
                    state = GTK_STATE_INSENSITIVE;
                else if (option->state & State_Sunken && (scrollBar->activeSubControls & SC_ScrollBarAddLine)) {
                    state = GTK_STATE_ACTIVE;
                    shadow = GTK_SHADOW_IN;
                } else if (option->state & State_MouseOver && (scrollBar->activeSubControls & SC_ScrollBarAddLine))
                    state = GTK_STATE_PRELIGHT;
                qgtkstyle_paint_box(gtkWindow, scrollbarWidget,
                                      horizontal ? "hscrollbar" : "vscrollbar", scrollBarAddLine,
                                      state, shadow, style, painter, "add");
                qgtkstyle_paint_arrow(gtkWindow, scrollbarWidget, horizontal ? "hscrollbar" : "vscrollbar", scrollBarAddLine.adjusted(4, 4, -4, -4), 
                                      horizontal ? GTK_ARROW_RIGHT : GTK_ARROW_DOWN, state, GTK_SHADOW_NONE, FALSE, style, painter);
            }

            if (scrollBar->subControls & SC_ScrollBarSubLine) {
                gtkVScrollBar->allocation.y = 0;
                gtkVScrollBar->allocation.height = scrollBarSubLine.height();
                gtkHScrollBar->allocation.x = 0;
                gtkHScrollBar->allocation.width = scrollBarSubLine.width();

                GtkShadowType shadow = GTK_SHADOW_OUT;
                GtkStateType state = GTK_STATE_NORMAL;
                if (!(option->state & State_Enabled) || (fakePos == 0))
                    state = GTK_STATE_INSENSITIVE;
                else if (option->state & State_Sunken && (scrollBar->activeSubControls & SC_ScrollBarSubLine)) {
                    shadow = GTK_SHADOW_IN;
                    state = GTK_STATE_ACTIVE;
                } else if (option->state & State_MouseOver && (scrollBar->activeSubControls & SC_ScrollBarSubLine))
                    state = GTK_STATE_PRELIGHT;

                qgtkstyle_paint_box(gtkWindow, scrollbarWidget, horizontal ? "hscrollbar" : "vscrollbar", scrollBarSubLine,
                                      state, shadow, style, painter, "sub", true);
                qgtkstyle_paint_arrow(gtkWindow, scrollbarWidget, horizontal ? "hscrollbar" : "vscrollbar", scrollBarSubLine.adjusted(4, 4, -4, -4), 
                                      horizontal ? GTK_ARROW_LEFT : GTK_ARROW_UP, state, GTK_SHADOW_NONE, FALSE, style, painter);
            }
        }
        break;
#endif //QT_NO_SCROLLBAR
#ifndef QT_NO_SPINBOX
    case CC_SpinBox:
        if (const QStyleOptionSpinBox *spinBox = qstyleoption_cast<const QStyleOptionSpinBox *>(option)) {
            GtkWidget *gtkSpinButton = get_gtk_spinbutton();
            bool isEnabled = (spinBox->state & State_Enabled);
            bool hover = isEnabled && (spinBox->state & State_MouseOver);
            bool sunken = (spinBox->state & State_Sunken);
            bool upIsActive = (spinBox->activeSubControls == SC_SpinBoxUp);
            bool downIsActive = (spinBox->activeSubControls == SC_SpinBoxDown);

            //### Move this to subControlRect
            QRect upRect = subControlRect(CC_SpinBox, option, SC_SpinBoxUp, widget);
            upRect.setTop(option->rect.top());
            upRect.setRight(option->rect.right());
            QRect downRect = subControlRect(CC_SpinBox, option, SC_SpinBoxDown, widget);
            downRect.setBottom(option->rect.bottom());
            downRect.setRight(option->rect.right());

            QRect updown = option->rect;
            updown.setLeft(upRect.left());
            QRect editArea = option->rect;
            editArea.setRight(updown.left());

            if (spinBox->frame) {
                get_gtk_spinbutton();

                GtkShadowType shadow = GTK_SHADOW_OUT;
                GtkStateType state = gtkState(option);
                if (!(option->state & State_Enabled))
                    state = GTK_STATE_INSENSITIVE;
                else if (option->state & State_HasFocus)
                    state = GTK_STATE_NORMAL;
                else if (state == GTK_STATE_PRELIGHT)
                    state = GTK_STATE_NORMAL;

                shadow = GTK_SHADOW_IN;
                style = get_style(gtkSpinButton);

                if (option->state & State_HasFocus)
                    GTK_WIDGET_SET_FLAGS(gtkSpinButton, GTK_HAS_FOCUS);
                else
                    GTK_WIDGET_UNSET_FLAGS(gtkSpinButton, GTK_HAS_FOCUS);

                qgtkstyle_paint_shadow(gtkWindow, gtkSpinButton, "entry", editArea, state, GTK_SHADOW_IN, gtkSpinButton->style,
                                         painter, option->state & State_HasFocus ? "f" : QString());

                int xt = style->xthickness;
                int yt = style->ythickness;
                painter->fillRect(option->rect.adjusted(xt, yt, -xt, -yt), option->palette.base().color());

                qgtkstyle_paint_box(gtkWindow, gtkSpinButton, "spinbutton", updown, state, GTK_SHADOW_OUT, style, painter);

                QRect upArrow = upRect.adjusted(-1, 0, 0, 0);
                QRect downArrow = downRect.adjusted(-1, 0, 0, 0);
                upRect.setSize(downRect.size());
                if (!(option->state & State_Enabled))
                    qgtkstyle_paint_box(gtkWindow, gtkSpinButton, "spinbutton_up", upRect, GTK_STATE_INSENSITIVE, GTK_SHADOW_IN, style, painter);
                else if (upIsActive && sunken)
                    qgtkstyle_paint_box(gtkWindow, gtkSpinButton, "spinbutton_up", upRect, GTK_STATE_ACTIVE, GTK_SHADOW_IN, style, painter);
                else if (upIsActive && hover)
                    qgtkstyle_paint_box(gtkWindow, gtkSpinButton, "spinbutton_up", upRect, GTK_STATE_PRELIGHT, GTK_SHADOW_OUT, style, painter);
                else
                    qgtkstyle_paint_box(gtkWindow, gtkSpinButton, "spinbutton_up", upRect, GTK_STATE_NORMAL, GTK_SHADOW_OUT, style, painter);

                if (!(option->state & State_Enabled))
                    qgtkstyle_paint_box(gtkWindow, gtkSpinButton, "spinbutton_down", downRect, GTK_STATE_INSENSITIVE, GTK_SHADOW_IN, style, painter);
                else if (downIsActive && sunken)
                    qgtkstyle_paint_box(gtkWindow, gtkSpinButton, "spinbutton_down", downRect, GTK_STATE_ACTIVE, GTK_SHADOW_IN, style, painter);
                else if (downIsActive && hover)
                    qgtkstyle_paint_box(gtkWindow, gtkSpinButton, "spinbutton_down", downRect, GTK_STATE_PRELIGHT, GTK_SHADOW_OUT, style, painter);
                else
                    qgtkstyle_paint_box(gtkWindow, gtkSpinButton, "spinbutton_down", downRect, GTK_STATE_NORMAL, GTK_SHADOW_OUT, style, painter);

            }

            if (spinBox->buttonSymbols == QAbstractSpinBox::PlusMinus) {
                int centerX = upRect.center().x();
                int centerY = upRect.center().y();
                // plus/minus
                if (spinBox->activeSubControls == SC_SpinBoxUp && sunken) {
                    painter->drawLine(1 + centerX - 2, 1 + centerY, 1 + centerX + 2, 1 + centerY);
                    painter->drawLine(1 + centerX, 1 + centerY - 2, 1 + centerX, 1 + centerY + 2);
                } else {
                    painter->drawLine(centerX - 2, centerY, centerX + 2, centerY);
                    painter->drawLine(centerX, centerY - 2, centerX, centerY + 2);
                }
                centerX = downRect.center().x();
                centerY = downRect.center().y();
                if (spinBox->activeSubControls == SC_SpinBoxDown && sunken) {
                    painter->drawLine(1 + centerX - 2, 1 + centerY, 1 + centerX + 2, 1 + centerY);
                } else {
                    painter->drawLine(centerX - 2, centerY, centerX + 2, centerY);
                }
            } else {
                // arrows
                GtkStateType state = GTK_STATE_NORMAL;
                if (!(option->state & State_Enabled) || !(spinBox->stepEnabled & QAbstractSpinBox::StepUpEnabled))
                    state = GTK_STATE_INSENSITIVE;
                qgtkstyle_paint_arrow(gtkWindow, gtkSpinButton, "spinbutton", upRect.adjusted(4, 3, -4, -3), GTK_ARROW_UP, state,
                                        GTK_SHADOW_NONE, FALSE, style, painter);
                state = GTK_STATE_NORMAL;
                if (!(option->state & State_Enabled) || !(spinBox->stepEnabled & QAbstractSpinBox::StepDownEnabled))
                    state = GTK_STATE_INSENSITIVE;
                qgtkstyle_paint_arrow(gtkWindow, gtkSpinButton, "spinbutton", downRect.adjusted(4, 3, -4, -3), GTK_ARROW_DOWN, state,
                                        GTK_SHADOW_NONE, FALSE, style, painter);
            }
        }
        break;
#endif // QT_NO_SPINBOX

#ifndef QT_NO_SLIDER
    case CC_Slider:
        if (const QStyleOptionSlider *slider = qstyleoption_cast<const QStyleOptionSlider *>(option)) {
            GtkWidget *hScaleWidget = get_gtk_hscale();
            GtkWidget *vScaleWidget = get_gtk_vscale();

            QRect groove = subControlRect(CC_Slider, option, SC_SliderGroove, widget);
            QRect handle = subControlRect(CC_Slider, option, SC_SliderHandle, widget);
            QRect ticks = subControlRect(CC_Slider, option, SC_SliderTickmarks, widget);

            bool horizontal = slider->orientation == Qt::Horizontal;
            bool ticksAbove = slider->tickPosition & QSlider::TicksAbove;
            bool ticksBelow = slider->tickPosition & QSlider::TicksBelow;
            QColor activeHighlight = option->palette.color(QPalette::Normal, QPalette::Highlight);
            QPixmap cache;

            QBrush oldBrush = painter->brush();
            QPen oldPen = painter->pen();

            QColor shadowAlpha(Qt::black);
            shadowAlpha.setAlpha(10);
            QColor highlightAlpha(Qt::white);
            highlightAlpha.setAlpha(80);

            GtkWidget *scaleWidget = horizontal ? hScaleWidget : vScaleWidget;
            style = scaleWidget->style;

            if ((option->subControls & SC_SliderGroove) && groove.isValid()) {
                GtkObject *adjustment =  p_gtk_adjustment_new(slider->sliderPosition,
                                         slider->minimum,
                                         slider->maximum,
                                         slider->singleStep,
                                         slider->singleStep,
                                         slider->pageStep);
                int outerSize;
                p_gtk_range_set_adjustment ((GtkRange*)(scaleWidget), (GtkAdjustment*)(adjustment));
                p_gtk_range_set_inverted((GtkRange*)(scaleWidget), !horizontal);
                p_gtk_widget_style_get (scaleWidget, "trough-border", &outerSize, NULL);
                outerSize++;

                GtkStateType state = gtkState(option);
                int focusFrameMargin = 2;
                QRect grooveRect = option->rect.adjusted(focusFrameMargin, outerSize + focusFrameMargin, 
                                                        -focusFrameMargin, -outerSize - focusFrameMargin);

                qgtkstyle_paint_box(gtkWindow, scaleWidget, "trough", grooveRect, state,
                                      GTK_SHADOW_IN, style, painter, QString("p%0").arg(slider->sliderPosition));

                gboolean trough_side_details; // Indicates if the upper or lower scale background differs
                p_gtk_widget_style_get ((GtkWidget*)(scaleWidget), "trough-side-details",   &trough_side_details, NULL);

                if (trough_side_details && horizontal) { //### Vertical sliders look broken with this for some reason
                    QRect lowerGroove = grooveRect;
                    lowerGroove.setRight(handle.center().x());
                    qgtkstyle_paint_box(gtkWindow, scaleWidget, "trough-lower", lowerGroove, state,
                                          GTK_SHADOW_IN, style, painter, QString("p%0").arg(slider->sliderPosition));
                }

            }
            if (option->subControls & SC_SliderTickmarks) {
                painter->setPen(darkOutline);
                int tickSize = pixelMetric(PM_SliderTickmarkOffset, option, widget);
                int available = pixelMetric(PM_SliderSpaceAvailable, slider, widget);
                int interval = slider->tickInterval;
                if (interval <= 0) {
                    interval = slider->singleStep;
                    if (QStyle::sliderPositionFromValue(slider->minimum, slider->maximum, interval,
                                                        available)
                            - QStyle::sliderPositionFromValue(slider->minimum, slider->maximum,
                                                              0, available) < 3)
                        interval = slider->pageStep;
                }
                if (interval <= 0)
                    interval = 1;

                int v = slider->minimum;
                int len = pixelMetric(PM_SliderLength, slider, widget);
                while (v <= slider->maximum + 1) {
                    if (v == slider->maximum + 1 && interval == 1)
                        break;
                    const int v_ = qMin(v, slider->maximum);
                    int pos = sliderPositionFromValue(slider->minimum, slider->maximum,
                                                      v_, (horizontal
                                                           ? slider->rect.width()
                                                           : slider->rect.height()) - len,
                                                      slider->upsideDown) + len / 2;
                    int extra = 2 - ((v_ == slider->minimum || v_ == slider->maximum) ? 1 : 0);

                    if (horizontal) {
                        if (ticksAbove)
                            painter->drawLine(pos, slider->rect.top() + extra,
                                              pos, slider->rect.top() + tickSize);
                        if (ticksBelow)
                            painter->drawLine(pos, slider->rect.bottom() - extra,
                                              pos, slider->rect.bottom() - tickSize);
                    } else {
                        if (ticksAbove)
                            painter->drawLine(slider->rect.left() + extra, pos,
                                              slider->rect.left() + tickSize, pos);
                        if (ticksBelow)
                            painter->drawLine(slider->rect.right() - extra, pos,
                                              slider->rect.right() - tickSize, pos);
                    }
                    // in the case where maximum is max int
                    int nextInterval = v + interval;
                    if (nextInterval < v)
                        break;
                    v = nextInterval;
                }
            }
            // draw handle
            if (option->subControls & SC_SliderHandle) {
                GtkShadowType shadow =  GTK_SHADOW_OUT;
                GtkStateType state = GTK_STATE_NORMAL;
                if (!(option->state & State_Enabled))
                    state = GTK_STATE_INSENSITIVE;
                else if (option->state & State_MouseOver && option->activeSubControls & SC_SliderHandle)
                    state = GTK_STATE_PRELIGHT;
                bool horizontal = option->state & State_Horizontal;

                if (slider->state & State_HasFocus) {
                    QStyleOptionFocusRect fropt;
                    fropt.QStyleOption::operator=(*slider);
                    fropt.rect = slider->rect.adjusted(-1, -1 ,1, 1);
                    if (horizontal) {
                        fropt.rect.setTop(handle.top() - 3);
                        fropt.rect.setBottom(handle.bottom() + 4);
                    } else {
                        fropt.rect.setLeft(handle.left() - 3);
                        fropt.rect.setRight(handle.right() + 3);
                    }
                    drawPrimitive(PE_FrameFocusRect, &fropt, painter, widget);
                }

                qgtkstyle_paint_slider(gtkWindow, scaleWidget, horizontal ? "hscale" : "vscale", handle, state, shadow, style, painter,
                                         horizontal ? GTK_ORIENTATION_HORIZONTAL : GTK_ORIENTATION_VERTICAL);
            }

            painter->setBrush(oldBrush);
            painter->setPen(oldPen);
        }
        break;
#endif // QT_NO_SLIDER
    default:
        QCleanlooksStyle::drawComplexControl(control, option, painter, widget);
        break;
    }
}


void QGtkStyle::drawControl(ControlElement element,
                            const QStyleOption *option,
                            QPainter *painter,
                            const QWidget *widget) const
{
    GtkStyle* style = getGtkStyle();
    GtkWidget *gtkWindow = get_gtk_window();
    if (!getGtkStyle()) {
        QCleanlooksStyle::drawControl(element, option, painter, widget);
        return;
    }

    switch (element) {
    case CE_PushButtonLabel:
        if (const QStyleOptionButton *button = qstyleoption_cast<const QStyleOptionButton *>(option)) {
            QRect ir = button->rect;
            uint tf = Qt::AlignVCenter | Qt::TextShowMnemonic;
            QPoint buttonShift;
            if (option->state & State_Sunken)
                buttonShift = QPoint(pixelMetric(PM_ButtonShiftHorizontal, option, widget),
                                     pixelMetric(PM_ButtonShiftVertical, option, widget));

            if (!styleHint(SH_UnderlineShortcut, button, widget))
                tf |= Qt::TextHideMnemonic;

            if (!button->icon.isNull()) {
                //Center both icon and text
                QPoint point;

                QIcon::Mode mode = button->state & State_Enabled ? QIcon::Normal
                                                              : QIcon::Disabled;
                if (mode == QIcon::Normal && button->state & State_HasFocus)
                    mode = QIcon::Active;
                QIcon::State state = QIcon::Off;
                if (button->state & State_On)
                    state = QIcon::On;

                QPixmap pixmap = button->icon.pixmap(button->iconSize, mode, state);
                int w = pixmap.width();
                int h = pixmap.height();

                if (!button->text.isEmpty())
                    w += button->fontMetrics.width(button->text) + 2;

                point = QPoint(ir.x() + ir.width() / 2 - w / 2,
                               ir.y() + ir.height() / 2 - h / 2);

                if (button->direction == Qt::RightToLeft)
                    point.rx() += pixmap.width();

                painter->drawPixmap(visualPos(button->direction, button->rect, point + buttonShift), pixmap);

                if (button->direction == Qt::RightToLeft)
                    ir.translate(-point.x() - 2, 0);
                else
                    ir.translate(point.x() + pixmap.width(), 0);
                
                // left-align text if there is
                if (!button->text.isEmpty())
                    tf |= Qt::AlignLeft;

            } else {
                tf |= Qt::AlignHCenter;
            }

            ir.translate(buttonShift);

            //this tweak ensures the font is perfectly centered on small sizes
            //but slightly downward to make it more gnomeish if not
            if (button->fontMetrics.height() > 14)
                ir.translate(0, 1);

            if (button->features & QStyleOptionButton::HasMenu)
                ir = ir.adjusted(0, 0, -pixelMetric(PM_MenuButtonIndicator, button, widget), 0);
            drawItemText(painter, ir, tf, button->palette, (button->state & State_Enabled),
                         button->text, QPalette::ButtonText);
        }
        break;
    case CE_HeaderSection:
        painter->save();
        // Draws the header in tables.
        if (const QStyleOptionHeader *header = qstyleoption_cast<const QStyleOptionHeader *>(option)) {
            Q_UNUSED(header);
            GtkWidget *gtkTreeHeader = get_gtk_tree_header();
            GtkStateType state = gtkState(option);
            GtkShadowType shadow = GTK_SHADOW_OUT;
            if (option->state & State_Sunken)
                shadow = GTK_SHADOW_IN;
            qgtkstyle_paint_box(gtkWindow, gtkTreeHeader, "button",  option->rect.adjusted(-1, 0, 0, 0), state, shadow, gtkTreeHeader->style, painter);
        }
        painter->restore();
        break;

#ifndef QT_NO_SIZEGRIP
    case CE_SizeGrip:
        qgtkstyle_resize_grip(gtkWindow, NULL, "window", option->rect, GTK_STATE_NORMAL,
                                GTK_SHADOW_OUT, GDK_WINDOW_EDGE_SOUTH_EAST, style, painter);
        break;
#endif // QT_NO_SIZEGRIP
 case CE_MenuBarEmptyArea:
        {
                        GtkWidget *gtkMenubar = get_gtk_menubar();
            if (widget) { // See CE_MenuBarItem
                QRect menuBarRect = widget->rect();
                QPixmap pixmap(menuBarRect.size());
                pixmap.fill(Qt::transparent);            
                QPainter pmPainter(&pixmap);
                GtkShadowType shadow_type;  
                gtk_widget_style_get (gtkMenubar, "shadow-type", &shadow_type, NULL);
                qgtkstyle_paint_box(gtkWindow, gtkMenubar, "menubar",  menuBarRect,
                                      GTK_STATE_NORMAL, shadow_type, gtkMenubar->style, &pmPainter);
                pmPainter.end();
                painter->drawPixmap(option->rect, pixmap, option->rect);
            }
        }
        break;
    case CE_MenuBarItem:
       painter->save();
       if (const QStyleOptionMenuItem *mbi = qstyleoption_cast<const QStyleOptionMenuItem *>(option)) {
            GtkWidget *gtkMenubarItem = get_gtk_menubaritem();
            GtkWidget *gtkMenubar = get_gtk_menubar();

            style = gtkMenubarItem->style;
            if (widget) {
                // Since Qt does not currently allow filling the entire background
                // we use a hack for this by making a complete menubar each time and
                // paint with the correct offset inside it. Pixmap caching should resolve
                // most of the performance penalty.
                QRect menuBarRect = widget->rect();
                QPixmap pixmap(menuBarRect.size());
                pixmap.fill(Qt::transparent);            
                QPainter pmPainter(&pixmap);
                qgtkstyle_paint_box(gtkWindow, gtkMenubar, "menubar",  menuBarRect,
                                      GTK_STATE_NORMAL, GTK_SHADOW_OUT, gtkMenubar->style, &pmPainter);
                pmPainter.end();
                painter->drawPixmap(option->rect, pixmap, option->rect);
            }
            QStyleOptionMenuItem item = *mbi;
            bool act = mbi->state & State_Selected && mbi->state & State_Sunken;
            bool dis = !(mbi->state & State_Enabled);
            item.rect = mbi->rect.adjusted(0, 1, 0, 1);
            get_gtk_menubaritem();
            GdkColor gdkText = gtkMenubarItem->style->fg[GTK_STATE_NORMAL];
            GdkColor gdkHText = gtkMenubarItem->style->fg[GTK_STATE_PRELIGHT];
            QColor normalTextColor = QColor(gdkText.red>>8, gdkText.green>>8, gdkText.blue>>8);
            QColor highlightedTextColor = QColor(gdkHText.red>>8, gdkHText.green>>8, gdkHText.blue>>8);
            item.palette.setBrush(QPalette::HighlightedText, highlightedTextColor);
            item.palette.setBrush(QPalette::Text, normalTextColor);
            item.palette.setBrush(QPalette::ButtonText, normalTextColor);
            QCommonStyle::drawControl(element, &item, painter, widget);
            if (act) {
                qgtkstyle_paint_box(gtkWindow, gtkMenubarItem, "menuitem",  option->rect.adjusted(0, 0, 0, 3),
                                      GTK_STATE_PRELIGHT, GTK_SHADOW_IN, gtkMenubarItem->style, painter);
                //draw text
                QPalette::ColorRole textRole = dis ? QPalette::Text : QPalette::HighlightedText;
                uint alignment = Qt::AlignCenter | Qt::TextShowMnemonic | Qt::TextDontClip | Qt::TextSingleLine;
                if (!QCleanlooksStyle::styleHint(SH_UnderlineShortcut, mbi, widget))
                    alignment |= Qt::TextHideMnemonic;
                drawItemText(painter, item.rect, alignment, item.palette, mbi->state & State_Enabled, mbi->text, textRole);
            }
        }
        painter->restore();
        break;

    case CE_Splitter: {
            qgtkstyle_paint_handle(gtkWindow, NULL, "splitter", option->rect,
                                     gtkState(option), GTK_SHADOW_NONE,
                                     !(option->state & State_Horizontal) ? GTK_ORIENTATION_HORIZONTAL : GTK_ORIENTATION_VERTICAL,
                                     style, painter);
        }
        break;

#ifndef QT_NO_TOOLBAR
    case CE_ToolBar:
        if (const QStyleOptionToolBar *toolbar = qstyleoption_cast<const QStyleOptionToolBar *>(option)) {
            QRect rect = option->rect;
            // There is a 1 pixel gap between toolbar lines in some styles (i.e Human)
            if (toolbar->positionWithinLine != QStyleOptionToolBar::End)
                rect.adjust(0, 0, 1, 0);
            GtkWidget *gtkToolbar = get_gtk_toolbar();
            GtkShadowType shadow_type;
            gtk_widget_style_get (gtkToolbar, "shadow-type", &shadow_type, NULL);
            qgtkstyle_paint_box(gtkWindow, gtkToolbar, "toolbar",  rect,
                                  GTK_STATE_NORMAL, shadow_type, gtkToolbar->style, painter);
        }
        break;
#endif // QT_NO_TOOLBAR
    case CE_MenuItem:
        painter->save();
        // Draws one item in a popup menu.
        if (const QStyleOptionMenuItem *menuItem = qstyleoption_cast<const QStyleOptionMenuItem *>(option)) {
            GtkWidget *gtkMenu = get_gtk_menu();
            GtkWidget *gtkMenuItem = get_gtk_menuitem();
            style = get_style(gtkMenuItem);
            QColor borderColor = option->palette.background().color().darker(160);
            QColor shadow = option->palette.dark().color();

            if (menuItem->menuItemType == QStyleOptionMenuItem::Separator) {
                GtkWidget *gtkMenuSeparator = get_gtk_menuseparator();
                painter->setPen(shadow.lighter(106));
                gboolean wide_separators;
                gint     separator_height;
	            guint    horizontal_padding;
	            gtk_widget_style_get (gtkMenuSeparator,
                                "wide-separators",    &wide_separators,
                                "separator-height",   &separator_height,
                                "horizontal-padding", &horizontal_padding,
                                NULL);
                if (wide_separators)
                    qgtkstyle_paint_box(gtkWindow, gtkMenuSeparator, "hseparator", 
                                        option->rect.adjusted(0, 0, 0, -1), GTK_STATE_NORMAL, GTK_SHADOW_NONE, gtkMenu->style, painter);
                else
                    qgtkstyle_paint_hline(gtkWindow, gtkMenuSeparator, "hseparator",
                                          menuItem->rect, GTK_STATE_NORMAL, gtkMenu->style,
                                          painter, option->rect.left() + horizontal_padding, option->rect.width() - 2*horizontal_padding, 2);
                painter->restore();
                break;
            }

            bool selected = menuItem->state & State_Selected && menuItem->state & State_Enabled;
            if (selected) {
                QRect rect = option->rect.adjusted(0, 0, -1, -1);
                qgtkstyle_paint_box(gtkWindow, gtkMenuItem, "menuitem", rect, GTK_STATE_PRELIGHT, GTK_SHADOW_OUT, style, painter);
            }
            bool checkable = menuItem->checkType != QStyleOptionMenuItem::NotCheckable;
            bool checked = menuItem->checked;
            bool sunken = menuItem->state & State_Sunken;
            bool enabled = menuItem->state & State_Enabled;
            bool ignoreCheckMark = false;
            gint checkSize;
            p_gtk_widget_style_get (get_gtk_check_menuitem(), "indicator-size", &checkSize, NULL);
            int checkcol = qMax(menuItem->maxIconWidth, qMax(20, checkSize));

#ifndef QT_NO_COMBOBOX
            if (qobject_cast<const QComboBox*>(widget))
                ignoreCheckMark = true; //ignore the checkmarks provided by the QComboMenuDelegate
#endif
            if (!ignoreCheckMark) {
                // Check
                QRect checkRect(option->rect.left() + 7, option->rect.center().y() - checkSize/2, checkSize, checkSize);
                checkRect = visualRect(menuItem->direction, menuItem->rect, checkRect);
                if (checkable && menuItem->icon.isNull()) {
                    if (menuItem->checkType & QStyleOptionMenuItem::Exclusive) {
                        // Radio button
                        GtkShadowType shadow = GTK_SHADOW_OUT;
                        GtkStateType state = gtkState(option);
                        if (sunken)                            
                            state = GTK_STATE_ACTIVE;
                        if (checked)
                            shadow = GTK_SHADOW_IN;
                        qgtkstyle_paint_option(gtkMenuItem, checkRect, state, shadow, gtkMenuItem->style, painter);
                    } else {
                        // Check box
                        if (menuItem->icon.isNull()) {
                            GtkShadowType shadow = GTK_SHADOW_OUT;
                            GtkStateType state = gtkState(option);
                            if (sunken)                            
                                state = GTK_STATE_ACTIVE;
                            if (checked)
                                shadow = GTK_SHADOW_IN;
                            qgtkstyle_paint_checkbox(gtkMenuItem, checkRect, state, shadow, gtkMenuItem->style, painter);
                        }
                    }
                }
            } else { //ignore checkmark
                if (menuItem->icon.isNull())
                    checkcol = 0;
                else
                    checkcol = menuItem->maxIconWidth;
            }

            // Text and icon, ripped from windows style
            bool dis = !(menuItem->state & State_Enabled);
            bool act = menuItem->state & State_Selected;
            const QStyleOption *opt = option;
            const QStyleOptionMenuItem *menuitem = menuItem;

            QPainter *p = painter;
            QRect vCheckRect = visualRect(opt->direction, menuitem->rect,
                                          QRect(menuitem->rect.x() + 3, menuitem->rect.y(),
                                                checkcol, menuitem->rect.height()));
            if (!menuItem->icon.isNull()) {
                QIcon::Mode mode = dis ? QIcon::Disabled : QIcon::Normal;
                if (act && !dis)
                    mode = QIcon::Active;
                QPixmap pixmap;

                int smallIconSize = pixelMetric(PM_SmallIconSize, option, widget);
                QSize iconSize(smallIconSize, smallIconSize);
#ifndef QT_NO_COMBOBOX
                if (const QComboBox *combo = qobject_cast<const QComboBox*>(widget))
                    iconSize = combo->iconSize();
#endif // QT_NO_COMBOBOX
                if (checked)
                    pixmap = menuItem->icon.pixmap(iconSize, mode, QIcon::On);
                else
                    pixmap = menuItem->icon.pixmap(iconSize, mode);

                int pixw = pixmap.width();
                int pixh = pixmap.height();

                QRect pmr(0, 0, pixw, pixh);
                pmr.moveCenter(vCheckRect.center());
                painter->setPen(menuItem->palette.text().color());
                if (checkable && checked) {
                    QStyleOption opt = *option;
                    if (act) {
                        QColor activeColor = mergedColors(option->palette.background().color(),
                                                          option->palette.highlight().color());
                        opt.palette.setBrush(QPalette::Button, activeColor);
                    }
                    opt.state |= State_Sunken;
                    opt.rect = vCheckRect;
                    drawPrimitive(PE_PanelButtonCommand, &opt, painter, widget);
                }
                painter->drawPixmap(pmr.topLeft(), pixmap);
            }

            GdkColor gdkText = gtkMenuItem->style->fg[GTK_STATE_NORMAL];
            GdkColor gdkDText = gtkMenuItem->style->fg[GTK_STATE_INSENSITIVE];
            GdkColor gdkHText = gtkMenuItem->style->fg[GTK_STATE_PRELIGHT];
            QColor textColor = QColor(gdkText.red>>8, gdkText.green>>8, gdkText.blue>>8);
            QColor disabledTextColor = QColor(gdkDText.red>>8, gdkDText.green>>8, gdkDText.blue>>8);
            QColor highlightedTextColor = QColor(gdkHText.red>>8, gdkHText.green>>8, gdkHText.blue>>8);
            if (selected) {
                painter->setPen(highlightedTextColor);
            } else {
                painter->setPen(textColor);
            }
            int x, y, w, h;
            menuitem->rect.getRect(&x, &y, &w, &h);
            int tab = menuitem->tabWidth;
            int xm = windowsItemFrame + checkcol + windowsItemHMargin;
            int xpos = menuitem->rect.x() + xm;

            QRect textRect(xpos, y + windowsItemVMargin, w - xm - windowsRightBorder - tab + 1, h - 2 * windowsItemVMargin);
            QRect vTextRect = visualRect(opt->direction, menuitem->rect, textRect);
            QString s = menuitem->text;
            if (!s.isEmpty()) {                    // draw text
                p->save();
                int t = s.indexOf(QLatin1Char('\t'));
                int text_flags = Qt::AlignVCenter | Qt::TextShowMnemonic | Qt::TextDontClip | Qt::TextSingleLine;
                if (!styleHint(SH_UnderlineShortcut, menuitem, widget))
                    text_flags |= Qt::TextHideMnemonic;
                // Draw shortcut right aligned
                text_flags |= Qt::AlignRight;
                if (t >= 0) {
                    int rightMargin = 12; // Hardcode for now
                    QRect vShortcutRect = visualRect(opt->direction, menuitem->rect,
                                                     QRect(textRect.topRight(), QPoint(menuitem->rect.right() - rightMargin, textRect.bottom())));
                    if (dis && styleHint(SH_EtchDisabledText, option, widget)) {
                        p->setPen(menuitem->palette.light().color());
                        p->drawText(vShortcutRect.adjusted(1, 1, 1, 1), text_flags, s.mid(t + 1));
                        p->setPen(disabledTextColor);
                    }
                    p->drawText(vShortcutRect, text_flags , s.mid(t + 1));
                    s = s.left(t);
                }

                text_flags &= ~Qt::AlignRight;
                text_flags |= Qt::AlignLeft;
                QFont font = menuitem->font;

                if (menuitem->menuItemType == QStyleOptionMenuItem::DefaultItem)
                    font.setBold(true);

                p->setFont(font);
                if (dis && styleHint(SH_EtchDisabledText, option, widget)) {
                    p->setPen(menuitem->palette.light().color());
                    p->drawText(vTextRect.adjusted(1, 1, 1, 1), text_flags, s.left(t));
                    p->setPen(disabledTextColor);
                }
                p->drawText(vTextRect, text_flags, s.left(t));
                p->restore();
            }

            // Arrow
            if (menuItem->menuItemType == QStyleOptionMenuItem::SubMenu) {// draw sub menu arrow
                QPoint buttonShift(pixelMetric(PM_ButtonShiftHorizontal, option, widget),
                                   pixelMetric(PM_ButtonShiftVertical, option, widget));
                int dim = (menuItem->rect.height() - 8) / 2;
                PrimitiveElement arrow;
                arrow = QApplication::isRightToLeft() ? PE_IndicatorArrowLeft : PE_IndicatorArrowRight;
                int xpos = menuItem->rect.left() + menuItem->rect.width() - 3 - dim;
                QRect  vSubMenuRect = visualRect(option->direction, menuItem->rect,
                                                 QRect(xpos, menuItem->rect.top() + 
                                                       menuItem->rect.height() / 2 - dim / 2, dim, dim));
                GtkStateType state = enabled ? (act ? GTK_STATE_PRELIGHT: GTK_STATE_NORMAL) : GTK_STATE_INSENSITIVE;
                qgtkstyle_paint_arrow(gtkWindow, gtkMenuItem, "menuitem", vSubMenuRect, GTK_ARROW_RIGHT, state, 
                                    GTK_SHADOW_NONE, FALSE, style, painter);
            }
        }
        painter->restore();
        break;

    case CE_PushButton:
        if (const QStyleOptionButton *btn = qstyleoption_cast<const QStyleOptionButton *>(option)) {
            drawControl(CE_PushButtonBevel, btn, painter, widget);
            QStyleOptionButton subopt = *btn;
            subopt.rect = subElementRect(SE_PushButtonContents, btn, widget);
            drawControl(CE_PushButtonLabel, &subopt, painter, widget);
            if (btn->state & State_HasFocus)
                qgtkstyle_paint_focus(gtkWindow, NULL, "button", option->rect.adjusted(3, 3, -3, -3), GTK_STATE_ACTIVE, style, painter);
        }
        break;

#ifndef QT_NO_TABBAR
    case CE_TabBarTabShape:
        if (const QStyleOptionTab *tab = qstyleoption_cast<const QStyleOptionTab *>(option)) {
            GtkWidget *gtkNotebook = get_gtk_notebook();
            style = get_style(gtkNotebook);

            QRect rect = option->rect;
            GtkShadowType shadow = GTK_SHADOW_OUT;

            GtkStateType state = GTK_STATE_ACTIVE;
            if (tab->state & State_Selected)
                state = GTK_STATE_NORMAL;

            bool first = tab->position == QStyleOptionTab::Beginning || tab->position == QStyleOptionTab::OnlyOneTab;
            bool last = tab->position == QStyleOptionTab::End || tab->position == QStyleOptionTab::OnlyOneTab;
            bool selected = (tab->state & State_Selected);
            int tabOverlap = 1;
            painter->save();
            switch (tab->shape) {
            case QTabBar::RoundedNorth:
//                painter->setClipRect(option->rect.adjusted(-1, 0, 0, 0));
                if (selected)
                    rect.adjust(0, 0, 0, -2);
                else
                    rect.adjust(first ? 0 : -tabOverlap, 3, last ? 0 : tabOverlap, -2);
                qgtkstyle_paint_extention(gtkWindow, gtkNotebook, "tab", rect, 
                                          state, shadow, GTK_POS_BOTTOM, style, painter);
                break;
            case QTabBar::RoundedSouth:
                if (selected)
                    rect.adjust(0, 0, 0, -1);
                else
                    rect.adjust(first ? 0 : -tabOverlap, 1, last ? 0 : tabOverlap, -4);
                qgtkstyle_paint_extention(gtkWindow, gtkNotebook, "tab", rect.adjusted(0, 1, 0, 0), 
                                          state, shadow, GTK_POS_TOP, style, painter);
                break;
            case QTabBar::RoundedWest:
                qgtkstyle_paint_extention(gtkWindow, gtkNotebook, "tab", rect, state, shadow, GTK_POS_RIGHT, style, painter);
                break;
            case QTabBar::RoundedEast:
                qgtkstyle_paint_extention(gtkWindow, gtkNotebook, "tab", rect, state, shadow, GTK_POS_LEFT, style, painter);
                break;
            default:
                QCleanlooksStyle::drawControl(element, option, painter, widget);
                break;
            }
            painter->restore();
        }
        break;
#endif //QT_NO_TABBAR
    case CE_RadioButton: //fall through
    case CE_CheckBox:
        QWindowsStyle::drawControl(element, option, painter, widget);
        break;
    case CE_ProgressBarGroove:
        if (const QStyleOptionProgressBar *bar = qstyleoption_cast<const QStyleOptionProgressBar *>(option)) {
            Q_UNUSED(bar);
            GtkWidget *gtkProgressBar = get_gtk_progressbar();
            GtkStateType state = gtkState(option);
            qgtkstyle_paint_box(gtkWindow, gtkProgressBar, "trough",  option->rect, state, GTK_SHADOW_IN, gtkProgressBar->style, painter);
        }
        break;
    case CE_ProgressBarContents:
        if (const QStyleOptionProgressBar *bar = qstyleoption_cast<const QStyleOptionProgressBar *>(option)) {
            GtkStateType state = gtkState(option);
            GtkWidget *gtkProgressBar = get_gtk_progressbar();
            style = gtkProgressBar->style;
            qgtkstyle_paint_box(gtkWindow, gtkProgressBar, "trough",  option->rect, state, GTK_SHADOW_IN, style, painter);
            int xt = style->xthickness - 1;
            int yt = style->ythickness - 1;
            QRect rect = bar->rect.adjusted(xt, yt, -xt, -yt);
            bool vertical = false;
            bool inverted = false;
            bool indeterminate = (bar->minimum == 0 && bar->maximum == 0);
            // Get extra style options if version 2
            if (const QStyleOptionProgressBarV2 *bar2 = qstyleoption_cast<const QStyleOptionProgressBarV2 *>(option)) {
                vertical = (bar2->orientation == Qt::Vertical);
                inverted = bar2->invertedAppearance;
            }
            // If the orientation is vertical, we use a transform to rotate
            // the progress bar 90 degrees clockwise.  This way we can use the
            // same rendering code for both orientations.
            if (vertical) {
                rect.translate(xt + 1,yt - 1);
                rect = QRect(rect.left(), rect.top(), rect.height(), rect.width()); // flip width and height
                QTransform m;
                m.translate(rect.height()-1, -1.0);
                m.rotate(90.0);
                painter->setTransform(m);
            }

            int maxWidth = rect.width() - 4;
            int minWidth = 4;
            qint64 progress = (qint64)qMax(bar->progress, bar->minimum); // workaround for bug in QProgressBar
            double vc6_workaround = ((progress - qint64(bar->minimum)) / double(qint64(bar->maximum) - qint64(bar->minimum))) * maxWidth;
            int progressBarWidth = (int(vc6_workaround) > minWidth ) ? int(vc6_workaround) : minWidth;
            int width = indeterminate ? maxWidth : progressBarWidth;
            bool reverse = (!vertical && (bar->direction == Qt::RightToLeft)) || vertical;
            if (inverted)
                reverse = !reverse;
            QRect progressBar;
            if (!indeterminate) {
                if (!reverse)
                    progressBar.setRect(rect.left() + 1, rect.top() + 1, width + 1, rect.height() - 2);
                else
                    progressBar.setRect(rect.right() - 1 - width, rect.top() + 1, width + 1, rect.height() - 2);
            } /*else {
                                Q_D(const QCleanlooksStyle);
                                int slideWidth = ((rect.width() - 4) * 2) / 3;
                                int step = ((d->animateStep * slideWidth) / d->animationFps) % slideWidth;
                                if ((((d->animateStep * slideWidth) / d->animationFps) % (2 * slideWidth)) >= slideWidth)
                                    step = slideWidth - step;
                                progressBar.setRect(rect.left() + 1 + step, rect.top() + 1,
                                                    slideWidth / 2, rect.height() - 3);
                            }*/

            qgtkstyle_paint_box(gtkWindow, gtkProgressBar, "bar",  progressBar, state, GTK_SHADOW_OUT, style, painter);
        }
        break;
    default:
        QCleanlooksStyle::drawControl(element, option, painter, widget);
    }
}

/*!
  \reimp
*/
QRect QGtkStyle::subControlRect(ComplexControl control, const QStyleOptionComplex *option,
                                SubControl subControl, const QWidget *widget) const
{
    QRect rect = QWindowsStyle::subControlRect(control, option, subControl, widget);
    GtkStyle* style = getGtkStyle();
    if (!style)
        return QCleanlooksStyle::subControlRect(control, option, subControl, widget);;


    switch (control) {

    case CC_Slider:
    if (const QStyleOptionSlider *slider = qstyleoption_cast<const QStyleOptionSlider *>(option)) {
        // Reserve space for outside focus rect
        QStyleOptionSlider sliderCopy = *slider;
        sliderCopy.rect = option->rect.adjusted(2, 2, -2, -2);
        return QCleanlooksStyle::subControlRect(control, &sliderCopy, subControl, widget);
    }
    break;        
#ifndef QT_NO_GROUPBOX
    case CC_GroupBox:
        if (qstyleoption_cast<const QStyleOptionGroupBox *>(option)) {
            rect = option->rect.adjusted(0, groupBoxTopMargin, 0, -groupBoxBottomMargin);
            int topMargin = 0;
            int topHeight = 0;
            topHeight = 10;
            QRect frameRect = rect;
            frameRect.setTop(topMargin);
            if (subControl == SC_GroupBoxFrame)
                return rect;
            else if (subControl == SC_GroupBoxContents) {
                int margin = 0;
                int leftMarginExtension = 8;
                return frameRect.adjusted(leftMarginExtension + margin, margin + topHeight + groupBoxTitleMargin, -margin, -margin);
            }
            if (const QGroupBox *groupBoxWidget = qobject_cast<const QGroupBox *>(widget)) {
                //Prepare metrics for a bold font
                QFont font = widget->font();
                font.setBold(true);
                QFontMetrics fontMetrics(font);
                QSize textRect = fontMetrics.boundingRect(groupBoxWidget->title()).size() + QSize(2, 4);
                int indicatorWidth = pixelMetric(PM_IndicatorWidth, option, widget);
                int indicatorHeight = pixelMetric(PM_IndicatorHeight, option, widget);
                if (subControl == SC_GroupBoxCheckBox) {
                    rect.setWidth(indicatorWidth);
                    rect.setHeight(indicatorHeight);
                    rect.moveTop((fontMetrics.height() - indicatorHeight) / 2 + groupBoxTopMargin);
                } else if (subControl == SC_GroupBoxLabel) {
                    if (groupBoxWidget->isCheckable())
                        rect.adjust(indicatorWidth + 4, 0, 0, 0);
                    rect.setSize(textRect);
                }
            }
        }
        return rect;
#endif
#ifndef QT_NO_TOOLBUTTON
    case CC_ToolButton:
        if (const QStyleOptionToolButton *tb = qstyleoption_cast<const QStyleOptionToolButton *>(option)) {
            int mbi = 12 + pixelMetric(PM_MenuButtonIndicator, tb, widget);
            rect = tb->rect;
            switch (subControl) {
            case SC_ToolButton:
                if ((tb->features
                        & (QStyleOptionToolButton::MenuButtonPopup | QStyleOptionToolButton::PopupDelay))
                        == QStyleOptionToolButton::MenuButtonPopup)
                    rect.adjust(0, 0, -mbi, 0);
                break;
            case SC_ToolButtonMenu:
                if ((tb->features
                        & (QStyleOptionToolButton::MenuButtonPopup | QStyleOptionToolButton::PopupDelay))
                        == QStyleOptionToolButton::MenuButtonPopup)
                    rect.adjust(rect.width() - mbi, 0, 0, 0);
                break;
            default:
                break;
            }
            rect = visualRect(tb->direction, tb->rect, rect);
        }
        break;
#endif // QT_NO_TOOLBUTTON
#ifndef QT_NO_SPINBOX
    case CC_SpinBox:
        if (const QStyleOptionSpinBox *spinbox = qstyleoption_cast<const QStyleOptionSpinBox *>(option)) {
            QSize bs;
            int center = spinbox->rect.height() / 2;
            int fw = spinbox->frame ? pixelMetric(PM_SpinBoxFrameWidth, spinbox, widget) : 0;
            int y = fw;
            bs.setHeight(qMax(8, spinbox->rect.height()/2 - y));
            bs.setWidth(15);
            int x, lx, rx;
            x = spinbox->rect.width() - y - bs.width() + 2;
            lx = fw;
            rx = x - fw;
            switch (subControl) {
            case SC_SpinBoxUp:
                if (spinbox->buttonSymbols == QAbstractSpinBox::NoButtons)
                    return QRect();
                rect = QRect(x, fw, bs.width(), center - fw);
                break;
            case SC_SpinBoxDown:
                if (spinbox->buttonSymbols == QAbstractSpinBox::NoButtons)
                    return QRect();

                rect = QRect(x, center, bs.width(), spinbox->rect.bottom() - center - fw + 1);
                break;
            case SC_SpinBoxEditField:
                if (spinbox->buttonSymbols == QAbstractSpinBox::NoButtons) {
                    rect = QRect(lx, fw, spinbox->rect.width() - 2*fw, spinbox->rect.height() - 2*fw);
                } else {
                    rect = QRect(lx, fw, rx - qMax(fw - 1, 0), spinbox->rect.height() - 2*fw);
                }
                break;
            case SC_SpinBoxFrame:
                rect = spinbox->rect;
            default:
                break;
            }
            rect = visualRect(spinbox->direction, spinbox->rect, rect);
        }
        break;
#endif // Qt_NO_SPINBOX
#ifndef QT_NO_COMBOBOX
    case CC_ComboBox:
        if (const QStyleOptionComboBox *box = qstyleoption_cast<const QStyleOptionComboBox *>(option)) {
            switch (subControl) {
            case SC_ComboBoxArrow:
                rect = visualRect(option->direction, option->rect, rect);
                rect.setRect(rect.right() - 18, rect.top() - 2,
                             19, rect.height() + 4);
                rect = visualRect(option->direction, option->rect, rect);
                if (box->editable)
                    rect.adjust(-2, 0, 0, 0);
                break;
            case SC_ComboBoxEditField: {
                    int frameWidth = 3;
                    rect = visualRect(option->direction, option->rect, rect);
                    rect.setRect(option->rect.left() + frameWidth, option->rect.top() + frameWidth,
                                 option->rect.width() - 19 - 2 * frameWidth,
                                 option->rect.height() - 2 * frameWidth);
                    if (!box->editable) {
                        rect.adjust(2, 0, 0, 0);
                        if (box->state & (State_Sunken | State_On))
                            rect.translate(1, 1);
                    } else
                        rect.adjust(0, 0, -3, 0);
                    rect = visualRect(option->direction, option->rect, rect);
                    break;
                }
            default:
                break;
            }
        }

        break;
#endif // QT_NO_COMBOBOX
    default:
        break;
    }
    return rect;
}

QSize QGtkStyle::sizeFromContents(ContentsType type, const QStyleOption *option,
                                  const QSize &size, const QWidget *widget) const
{

    QSize newSize = QCleanlooksStyle::sizeFromContents(type, option, size, widget);
    GtkStyle* style = getGtkStyle();
    if (!style)
        return newSize;

    switch (type) {
    case CT_ToolButton:
        if (!(widget && qobject_cast<QToolBar *>(widget->parentWidget())))
            newSize.setHeight(qMax(newSize.height(), 31)); // When not in a toolbar, make it aligned with normal buttons
        if (const QStyleOptionToolButton *toolbutton = qstyleoption_cast<const QStyleOptionToolButton *>(option)) {
            if (toolbutton->icon.isNull())
                newSize += QSize(0, -3);
            if (toolbutton->subControls & SC_ToolButtonMenu)
                newSize += QSize(4, 0);
        }
        break;
    case CT_MenuItem:
        if (const QStyleOptionMenuItem *menuItem = qstyleoption_cast<const QStyleOptionMenuItem *>(option)) {
            int textMargin = 8;
            if (menuItem->menuItemType == QStyleOptionMenuItem::Separator) {
                GtkWidget *gtkMenuSeparator = get_gtk_menuseparator();
                gboolean wide_separators;
                gint     separator_height;
	            gtk_widget_style_get (gtkMenuSeparator,
                                "wide-separators",    &wide_separators,
                                "separator-height",   &separator_height,
                                NULL);                                
                newSize = QSize(size.width(), wide_separators ? separator_height - 1 : 7 );

                break;
            }
            GtkWidget *gtkMenuItem = get_gtk_menuitem();
            GtkStyle* style = get_style(gtkMenuItem);
            newSize += QSize(textMargin + style->xthickness - 2, style->ythickness - 4);
            // Cleanlooks assumes a check column of 20 pixels so we need to
            // expand it a bit
            gint checkSize;
            p_gtk_widget_style_get (get_gtk_check_menuitem(), "indicator-size", &checkSize, NULL);
            newSize.setHeight(qMax(newSize.height(), checkSize + 2));
            newSize.setWidth(newSize.width() + qMax(0, checkSize - 20));
        }
        break;
    case CT_Menu: {
            // This is evil, but QMenu adds 1 pixel too much
            newSize -= QSize(0, 1);
        }
        break;
    case CT_PushButton: {
            GtkWidget *gtkButton = get_gtk_button();
            GtkStyle* style = get_style(gtkButton);
            newSize += QSize(2 + style->xthickness, style->ythickness - 1);
            newSize.setHeight(qMax(newSize.height(), 31));
        }
        break;
    case CT_Slider: {
            newSize += QSize(4, 4);
        }
        break;
    case CT_MenuBarItem://cleanlooks adds 2 pixels
        newSize = QWindowsStyle::sizeFromContents(type, option, size, widget);
        break;
    case CT_ComboBox:
        if (!(widget && qobject_cast<QToolBar *>(widget->parentWidget())))
            newSize += QSize(0, 3);
        break;
    case CT_GroupBox:
        newSize += QSize(0, groupBoxBottomMargin + groupBoxTopMargin + groupBoxTitleMargin); // Add some space below the groupbox
        break;
    case CT_TabBarTab:
        newSize += QSize(1, 0);
        break;
    default:
        break;
    }
    return newSize;
}

QPixmap getIcon(const char* iconName, GtkIconSize size = GTK_ICON_SIZE_BUTTON) {
     GtkStyle *style = getGtkStyle();
     GtkIconSet* iconSet  = p_gtk_icon_factory_lookup_default (iconName);
     GdkPixbuf* icon = p_gtk_icon_set_render_icon(iconSet, style, GTK_TEXT_DIR_LTR, GTK_STATE_NORMAL, size, NULL, "button");
     uchar* data = (uchar*)gdk_pixbuf_get_pixels(icon);
     int index = 0;
     QImage converted(gdk_pixbuf_get_width(icon), gdk_pixbuf_get_height(icon), QImage::Format_ARGB32); \
     uchar* tdata = (uchar*)converted.bits();
     for ( int y = 0 ; y < gdk_pixbuf_get_height(icon) ; ++y) {
         for ( int x = 0 ; x < gdk_pixbuf_get_width(icon) ; ++x) {
              //int index = y * rowstride + x;
             tdata[index] = data[index + RED];
             tdata[index] = data[index + BLUE];
             tdata[index + GREEN] = data[index + GREEN];
             tdata[index + BLUE] = data[index + RED];
             tdata[index + ALPHA] = data[index + ALPHA];
             index += 4;
         }
     }
     gdk_pixbuf_unref(icon);
     // should we free iconset?
     return QPixmap::fromImage(converted);
 
}

/*! \reimp */
QPixmap QGtkStyle::standardPixmap(StandardPixmap sp, const QStyleOption *option,
                                     const QWidget *widget) const
{
    QPixmap pixmap;
    GtkStyle *style = getGtkStyle();
    if (!style)
        return QCleanlooksStyle::standardPixmap(sp, option, widget);
#ifdef QT_NO_IMAGEFORMAT_PNG
    Q_UNUSED(option);
    Q_UNUSED(widget);
#endif
    switch (sp) {
    case SP_TitleBarNormalButton:
        {
          QImage restoreButton((const char **)dock_widget_restore_xpm);
          QColor alphaCorner = restoreButton.color(2);
          alphaCorner.setAlpha(80);
          restoreButton.setColor(2, alphaCorner.rgba()); 
          alphaCorner.setAlpha(180);
          restoreButton.setColor(4, alphaCorner.rgba()); 
          return QPixmap::fromImage(restoreButton);
        }
        break;
    case SP_TitleBarCloseButton:
    case SP_DockWidgetCloseButton:
        {    
            QImage closeButton((const char **)dock_widget_close_xpm);
            QColor alphaCorner = closeButton.color(2);
            alphaCorner.setAlpha(80);
            closeButton.setColor(2, alphaCorner.rgba()); 
            return QPixmap::fromImage(closeButton);
        }
        break;
    case SP_ArrowLeft: {
        return getIcon("gtk-go-back");
    }
    case SP_ArrowRight: {
        return getIcon("gtk-go-forward");
    }
#if (QT_VERSION >= 0x040400)
    case SP_BrowserReload: {
        return getIcon("gtk-refresh");
    }
    case SP_BrowserStop: {
        return getIcon("gtk-stop");
    }
#endif
    case SP_DialogOkButton: {
        return getIcon(GTK_STOCK_OK);
    }
    case SP_DialogCancelButton: {
        return getIcon(GTK_STOCK_CANCEL);
    }
    case SP_DialogYesButton: {
        return getIcon(GTK_STOCK_YES);
    }
    case SP_DialogNoButton: {
        return getIcon(GTK_STOCK_NO);
    }
    case SP_DialogOpenButton: {
        return getIcon(GTK_STOCK_OPEN);
    }
    case SP_DialogCloseButton: {
        return getIcon(GTK_STOCK_CLOSE);
    }
    case SP_DialogApplyButton: {
        return getIcon(GTK_STOCK_APPLY);
    }
    case SP_DialogSaveButton: {
        return getIcon(GTK_STOCK_SAVE);
    }
    case SP_MessageBoxWarning: {
        return getIcon(GTK_STOCK_DIALOG_WARNING, GTK_ICON_SIZE_DIALOG);
    }
    case SP_MessageBoxQuestion: {
        return getIcon(GTK_STOCK_DIALOG_QUESTION, GTK_ICON_SIZE_DIALOG);
    }
    case SP_MessageBoxInformation: {
        return getIcon(GTK_STOCK_DIALOG_INFO, GTK_ICON_SIZE_DIALOG);
    }
    case SP_MessageBoxCritical: {
        return getIcon(GTK_STOCK_DIALOG_ERROR, GTK_ICON_SIZE_DIALOG);
    }
    default:
        return QCleanlooksStyle::standardPixmap(sp, option, widget);
    }
    return QPixmap();
}

/*! \reimp */
QRect QGtkStyle::subElementRect(SubElement element, const QStyleOption *option, const QWidget *widget) const
{
    QRect r = QCleanlooksStyle::subElementRect(element, option, widget);
    switch (element) {
    default:
        break;
    }
    return r;
}

